program main

  use vtype_module

  use fi_module
  use cia_module

  implicit none

  character(extstr) :: CtlFN  = 'conv_cia2nc.conf'

  character(extstr) :: CIAFileName
  character(extstr) :: OutputFileName
  character(extstr*2) :: Src
  character(extstr  ) :: Ref


  ! Local variables
  !

  integer , parameter   :: MaxNmlArySize = 256


  type( CIAParam ) :: CIAP

  INTEGER, PARAMETER :: nS = 3000 !c-mm  Number of wavenumbers of data available
  INTEGER, PARAMETER :: nT = 9    !c-mm  Number of temperatures of data available

  REAL(8) :: wn_arr(nS)
  REAL(8) :: temp_arr(nT)
  REAL(8) :: dim_arr(nS,nT)

  integer              :: NWaveNum
  integer              :: NTemp
  REAL(8), allocatable :: w_WaveNum(:)
  REAL(8), allocatable :: t_Temp   (:)
  REAL(8), allocatable :: wt_Coef  (:,:)

  integer            :: FileUnit
  integer            :: iostat

  integer            :: i
  integer            :: j
  integer            :: is
  integer            :: ie


  namelist /file_nml/ &
    & CIAFileName, &
    & OutputFileName, &
    & Src, &
    & Ref


  CIAFileName    = "../data_hitran/CIA/data/CO2-CO2_2011.cia"
  OutputFileName = "./out"


  ! Explanation

  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = file_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in file_nml is something wrong.'
  end if
  write( 6, nml = file_nml )
  !
  close( FileUnit )


  write( 6, * ) 'Input  File: ', trim( CIAFileName )
  write( 6, * ) 'Output File: ', trim( OutputFileName )


  call read_baranov_data(wn_arr,temp_arr,dim_arr)

  do i = 1, nS
    if ( 1200.0d0 <= wn_arr(i) ) exit
  end do
  is = i-1
  do i = nS, 1, -1
    if ( wn_arr(i) <= 1500.0d0 ) exit
  end do
  ie = i+1

  NWaveNum = ie - is + 1
  NTemp    = nT
  allocate( w_WaveNum( NWaveNum ) )
  allocate( t_Temp   ( NTemp    ) )
  allocate( wt_Coef  ( NWaveNum, NTemp ) )
  do i = 1, NWaveNum
    w_WaveNum(i) = wn_arr(i-1+is)
  end do
  do j = 1, NTemp
    t_Temp(j) = temp_arr(j)
  end do
  do j = 1, NTemp
    do i = 1, NWaveNum
      wt_Coef(i,j) = dim_arr(i-1+is,j)
    end do
  end do


  ! Unit conversion
  !
  !   conversion from cm-1 to m-1
  w_WaveNum = w_WaveNum * 1.0d2
  !
  !   conversion from cm-1 amagat-2 to cm5 molecule-2
  wt_Coef = wt_Coef * 1.385d-39
  !   conversion from cm5 molecule-2 to m5 molecule-2
  wt_Coef = wt_Coef * 1.0d-10

  call CIAStoreDataInObj( &
    & 2, 2, &
    & NWaveNum, NTemp, w_WaveNum, t_Temp, wt_Coef, &
    & CIAP  &
    & )

!!$  call CIAReadFile( CIAFileName, CIAP )
  call CIAWriteFileNetcdf( OutputFileName, Src, Ref, CIAP )
  call CIAParamFinalize( CIAP )


end program main
