program main

  use vtype_module

  use fi_module
  use cia_module

  implicit none

  character(extstr) :: CtlFN  = 'conv_cia2nc.conf'

  character(extstr) :: CIAFileName
  character(extstr) :: OutputFileName
  character(extstr) :: Src
  character(extstr) :: Ref

  real(dp) :: WaveNumS
  real(dp) :: WaveNumE

  ! Local variables
  !

  integer , parameter   :: MaxNmlArySize = 256


  type( CIAParam ) :: CIAP
  type( CIAParam ) :: CIAPOut

  integer               :: NWaveNum
  integer               :: iWaveNumS
  integer               :: iWaveNumE
  integer               :: NTemp
  real(dp), allocatable :: w_WaveNum(:)
  real(dp), allocatable :: wt_Coef  (:,:)

  integer            :: FileUnit
  integer            :: iostat

  integer            :: i
  integer            :: l


  namelist /wn_nml/ &
    & WaveNumS, &
    & WaveNumE

  namelist /file_nml/ &
    & CIAFileName, &
    & OutputFileName, &
    & Src, &
    & Ref


  WaveNumS = -1.0d0
  WaveNumE = -1.0d0

  CIAFileName    = "../data_hitran/CIA/data/CO2-CO2_2011.cia"
  OutputFileName = "./out"


  ! Explanation

  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = wn_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in wn_nml is something wrong.'
  end if
  write( 6, nml = wn_nml )
  !
  rewind( FileUnit )
  read( FileUnit, nml = file_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in file_nml is something wrong.'
  end if
  write( 6, nml = file_nml )
  !
  close( FileUnit )


  write( 6, * ) 'Input  File: ', trim( CIAFileName )
  write( 6, * ) 'Output File: ', trim( OutputFileName )

  call CIAReadHITRANCIAFile( CIAFileName, CIAP )

  if ( WaveNumS < 0.0d0 ) then
    iWaveNumS = 1
  else
    do l = 1, CIAP%NWaveNum
      if ( WaveNumS <= CIAP%w_WaveNum(l) ) then
        exit
      end if
    end do
    iWaveNumS = l
  end if
  if ( WaveNumE < 0.0d0 ) then
    iWaveNumE = CIAP%NWaveNum
  else
    do l = CIAP%NWaveNum, 1, -1
      if ( CIAP%w_WaveNum(l) <= WaveNumE ) then
        exit
      end if
    end do
    iWaveNumE = l
  end if

  NWaveNum = iWaveNumE - iWaveNumS + 1
  NTemp    = CIAP%NTemp
  allocate( w_WaveNum( NWaveNum ) )
  allocate( wt_Coef  ( NWaveNum, NTemp ) )
  do l = iWaveNumS, iWaveNumE
    w_WaveNum(l-iWaveNumS+1) = CIAP%w_WaveNum(l)
  end do
  do i = 1, NTemp
    do l = iWaveNumS, iWaveNumE
      wt_Coef(l-iWaveNumS+1,i) = CIAP%wt_Coef(l,i)
    end do
  end do
  call CIAStoreDataInObj( &
    & CIAP%IMol, CIAP%IMolOpp, &
    & NWaveNum, NTemp, w_WaveNum, CIAP%t_Temp, wt_Coef, &
    & CIAPOut  &
    & )
  deallocate( w_WaveNum )
  deallocate( wt_Coef   )

  call CIAWriteFileNetcdf( OutputFileName, Src, Ref, CIAPOut )

  call CIAParamFinalize( CIAP    )
  call CIAParamFinalize( CIAPOut )


end program main
