program main

  use vtype_module

  use fi_module
  use ni3_module

  implicit none

  character(extstr) :: CtlFN  = 'conv_ref2nc.conf'

  character(extstr) :: RefFileListName
  character(extstr) :: OutputDirName

  character(extstr) :: RefFileName
  character(extstr) :: OutputFileName

  character(extstr) :: Origin

  character(extstr) :: Src
  character(extstr) :: Ref

  character(extstr) :: Line

  integer :: lBlankLine
  integer :: lFirstData

  integer , parameter   :: MaxNmlArySize = 256


  integer              :: NWaveNum
  real(8), allocatable :: w_WaveLen  (:)
  real(8), allocatable :: w_ReflectWL(:)
  real(8), allocatable :: w_WaveNum  (:)
  real(8), allocatable :: w_ReflectWN(:)

  real(8), allocatable :: w_Array1(:)
  real(8), allocatable :: w_Array2(:)

  integer            :: FileListFileUnit

  integer            :: FileUnit
  integer            :: iostat

  integer            :: NFile
  integer            :: iFile

  integer            :: NLine

  integer                        :: NcID
  character(stdstr)              :: Mode
  integer                        :: NDims
  character(stdstr), allocatable :: a_DimNames(:)
  character(stdstr)              :: Name
  character(stdstr)              :: StdName
  character(stdstr)              :: LongName
  character(stdstr)              :: Units

  integer            :: l


  logical :: FlagBlankLine


  namelist /file_nml/ &
    & RefFileListName, &
    & OutputDirName


  RefFileListName = "filelist.txt"
  OutputDirName   = "./out"


  ! Explanation

  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = file_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in file_nml is something wrong.'
  end if
  write( 6, nml = file_nml )
  !
  close( FileUnit )


  write( 6, * ) 'File list  : ', trim( RefFileListName )

  NFile = fi_lc( RefFileListName )

  call fi_open( RefFileListName, "read", FileListFileUnit )

  do iFile = 1, NFile
    read( FileListFileUnit, '(a)' ) RefFileName
    if ( RefFileName(1:1) == '#' ) then
      write( 6, * ) 'Skip File  : ', trim( RefFileName )
      cycle
    end if

    write( 6, * ) '-------------------'
    write( 6, * ) 'Input File : ', trim( RefFileName )


    NLine = fi_lc( RefFileName )

    lBlankLine = 0

    call fi_open( RefFileName, "read", FileUnit )
    do l = 1, NLine
      Line = ''
      read( FileUnit, '(a)' ) Line
      call ChkBlankLine( Line, FlagBlankLine )
      if ( FlagBlankLine ) then
        lBlankLine = l
      end if
      if ( Line(1:5) == 'Name:' ) then
        OutputFileName = ''
        OutputFileName = Line(6:len_trim(Line))
        call DeleteBlank( OutputFileName )
        OutputFileName = trim( OutputFileName ) // '.nc'
      end if
      if ( Line(1:19) == 'Number of X Values:' ) then
        read( Line(20:len_trim(Line)), * ) NWaveNum
      end if
      if ( Line(1:7) == 'Origin:' ) then
        read( Line(8:len_trim(Line)), '(a)' ) Origin
      end if
    end do
    close( FileUnit )

    lFirstData = lBlankLine + 1

    OutputFileName = trim( OutputDirName ) // '/' // trim( OutputFileName )
    write( 6, * ) 'Output File: ', trim( OutputFileName )

    ! check NWaveNum
    if ( ( NLine-lFirstData+1 ) /= NWaveNum ) then
      write( 6, * ) "***'Number of X Values' of ", NWaveNum, " looks inappropriate."
      write( 6, * ) "***Accurate number would be ", NLine-lFirstData+1, "."
      write( 6, * ) "***Data start at line ", lFirstData, "."
      NWaveNum = NLine - lFirstData + 1
    end if

    allocate( w_WaveLen  ( NWaveNum ) )
    allocate( w_ReflectWL( NWaveNum ) )
    allocate( w_WaveNum  ( NWaveNum ) )
    allocate( w_ReflectWN( NWaveNum ) )


    call fi_open( RefFileName, "read", FileUnit )
    do l = 1, NLine-NWaveNum
      read( FileUnit, * )
    end do
    do l = 1, NWaveNum
      read( FileUnit, * ) w_WaveLen(l), w_ReflectWL(l)
    end do
    close( FileUnit )

    w_WaveLen   = w_WaveLen * 1.0d-6
    w_ReflectWL = w_ReflectWL * 1.0d-2

    w_WaveNum   = 1.0d0 / w_WaveLen
    w_ReflectWN = w_ReflectWL

    if ( w_WaveLen(1) > w_WaveLen(2) ) then
      allocate( w_Array1( NWaveNum ) )
      allocate( w_Array2( NWaveNum ) )
      w_Array1 = w_WaveLen
      w_Array2 = w_ReflectWL
      do l = 1, NWaveNum
        w_WaveLen  (l) = w_Array1(NWaveNum-l+1)
        w_ReflectWL(l) = w_Array2(NWaveNum-l+1)
      end do
      deallocate( w_Array1 )
      deallocate( w_Array2 )
    end if
    if ( w_WaveNum(1) > w_WaveNum(2) ) then
      allocate( w_Array1( NWaveNum ) )
      allocate( w_Array2( NWaveNum ) )
      w_Array1 = w_WaveNum
      w_Array2 = w_ReflectWN
      do l = 1, NWaveNum
        w_WaveNum  (l) = w_Array1(NWaveNum-l+1)
        w_ReflectWN(l) = w_Array2(NWaveNum-l+1)
      end do
      deallocate( w_Array1 )
      deallocate( w_Array2 )
    end if


    ! Output
    !

    Src = "ASTER Version 2.0; " // trim( Origin )
    Ref = "Baldridge et al. (2009), The ASTER spectral library version 2.0, Remote Sensing of Environment, 113, 711."


    Mode = 'new'
    call ni3_open( OutputFileName, Mode, NcID )
    !
    call ni3_set_ga( NcID, src = Src, ref = Ref )
    !
    Name     = "WaveNum"
    StdName  = Name
    LongName = 'wavenumber'
    Units    = 'm-1'
    call ni3_set_dim( NcID, Name, NI3_DOUBLE, w_WaveNum, &
      & StdName, LongName, Units )
    !
    Name     = "WaveLen"
    StdName  = Name
    LongName = 'wavenumber'
    Units    = 'm'
    call ni3_set_dim( NcID, Name, NI3_DOUBLE, w_WaveLen, &
      & StdName, LongName, Units )
    !
    NDims    = 1
    allocate( a_DimNames( NDims ) )
    a_DimNames(1) = 'WaveNum'
    !
    Name     = "Reflectance"
    StdName  = Name
    LongName = 'reflectance'
    Units    = '1'
    call ni3_def_var( NcID, Name, NI3_DOUBLE, NDims, a_DimNames, &
      & StdName, LongName, Units )
    call ni3_put_var( NcID, Name, w_ReflectWN )
    !
    deallocate( a_DimNames )
    !
    !
    NDims    = 1
    allocate( a_DimNames( NDims ) )
    a_DimNames(1) = 'WaveLen'
    !
    Name     = "ReflectanceWL"
    StdName  = Name
    LongName = 'reflectance'
    Units    = '1'
    call ni3_def_var( NcID, Name, NI3_DOUBLE, NDims, a_DimNames, &
      & StdName, LongName, Units )
    call ni3_put_var( NcID, Name, w_ReflectWL )
    !
    deallocate( a_DimNames )
    !
    !
    call ni3_close( NcID )


    deallocate( w_WaveLen   )
    deallocate( w_ReflectWL )
    deallocate( w_WaveNum   )
    deallocate( w_ReflectWN )

  end do

  close( FileListFileUnit )


contains

  !----------------------------------------------------------------------------

  subroutine ChkBlankLine( Line, FlagBlankLine )

    character(*), intent(in ) :: Line
    logical     , intent(out) :: FlagBlankLine


    ! Local variables
    !
    integer                       :: l


    FlagBlankLine = .true.


    do l = 1, len_trim( Line )
      if ( &
        & ( Line(l:l) /= ' ' ) .and. &
        & ( Line(l:l) /= char(9) ) &
        & ) then
        FlagBlankLine = .false.
      end if
    end do


  end subroutine ChkBlankLine

  !----------------------------------------------------------------------------

  subroutine DeleteBlank( Line )

    character(*), intent(inout) :: Line

    ! Local variables
    !
    character(len=len_trim(Line)) :: LineBK
    integer                       :: l
    integer                       :: m


    LineBK = Line
    Line   = ''

    m = 1
    do l = 1, len_trim( LineBK )
      if ( &
        & ( LineBK(l:l) /= ' ' ) .and. &
        & ( LineBK(l:l) /= '/' ) &
        & ) then
        Line(m:m) = LineBK(l:l)
        m = m + 1
      end if
    end do


  end subroutine DeleteBlank

end program main
