program main

  use vtype_module
  use netcdf
  use ni3_module

  use fi_module

  use set_1d_profile, only : Set1DProfileAtm
  use interpolate, only : Interpolate1D

  use io_profile, only : IOProfileOutput

  use hitranconst, only : &
    & NHITRANMolNum, &
    & m_HITRANMolName

  implicit none

  integer                 :: NcID
  character(extstr)       :: Mode


!!$  real(dp)                :: TempConst

  real(dp)                :: Ps
  real(dp)                :: Ts

  integer                 :: NLev

!!$  integer                 :: NLevTmp
!!$  real(dp), allocatable   :: r_PressTmp(:)

  real(dp), allocatable   :: r_Press(:)
  real(dp), allocatable   :: r_Height(:)
  integer                 :: NMolNum
  integer , allocatable   :: m_MolNum(:)             ! Molecular number

  real(dp), allocatable   :: r_Temp       (:)
  real(dp), allocatable   :: rm_VMRDryBase(:,:)
  real(dp), allocatable   :: rm_VMR       (:,:)

  real(dp), allocatable   :: r_VMRDryBaseH2O(:)
  real(dp), allocatable   :: r_VMRH2O       (:)

  character(stdstr) :: VarName

  logical :: FlagVarExist
  logical :: FlagVarSpecified

  integer :: iMolNum

  character(128) :: ctlfn = 'mkprofile.conf'

  character(128) :: InNcFn
  character(128) :: OutNcFn
  character(128) :: PressName
  character(128) :: TempName


  integer             , parameter   :: NChar = stdstr
  real(8)             , allocatable :: z_Press      (:)
  real(8)             , allocatable :: z_Height     (:)
  integer                           :: NPtcl
  integer             , allocatable :: a_PtclNum    (:)
  character(len=NChar), allocatable :: a_PtclName   (:)
  real(8)             , allocatable :: a_PtclDens   (:)
  real(8)             , allocatable :: za_PtclEffRad(:,:)
  real(8)             , allocatable :: za_PtclMMR   (:,:)


  integer            :: NMol

  integer, parameter :: MaxNmlArySize = 256
  integer            :: MolNum(MaxNmlArySize)
  real(8)            :: VMR   (MaxNmlArySize)

  integer              :: MolNumTmp
  real(8), allocatable :: r_VMRTmp(:)

  integer :: FileUnit
  integer :: iostat

  integer :: k
  integer :: m
  integer :: n


  namelist /file_nml/ &
    & InNcFn, OutNcFn, &
    & PressName, TempName

  namelist /atmosphere_nml/ &
!!$    & TempConst, &
    & MolNum, &
    & VMR

!!$  TempConst      = -1.0d0

  InNcFn      = ''
  OutNcFn     = ''
  !
  MolNum(:)   = -1
  VMR   (:)   = -1.0d0


  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = file_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist, file_nml, is something wrong.'
  end if
  write( 6, nml = file_nml )
  !
  rewind( FileUnit )
  read( FileUnit, nml = atmosphere_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist, atmosphere_nml, is something wrong.'
  end if
  write( 6, nml = atmosphere_nml )
  !
  close( FileUnit )

  do m = 1, MaxNmlArySize
    if ( MolNum(m) <= 0 ) exit
    if ( VMR   (m) <  0.0d0 ) then
      write( 6, * ) 'Volume mixing ratio specified in namelist is negative for m = ', m
      stop
    end if
  end do
  NMol = min( m-1, MaxNmlArySize )


  !
  ! Input
  !
  Mode = "read"
  call ni3_open( InNcFN, Mode, NcID )
  !
  call ni3_inq_dimlen( NcID, PressName, NLev )
  !
  NMolNum = NMol
  do m = 1, NHITRANMolNum
    VarName = 'VMR' // trim( m_HITRANMolName(m) )
    FlagVarExist = ni3_chk_var( NcID, VarName )
    !
    if ( FlagVarExist ) then
      FlagVarSpecified = .false.
      do n = 1, NMol
        if ( m == MolNum(n) ) then
          FlagVarSpecified = .true.
          exit
        end if
      end do
      if ( .not. FlagVarSpecified ) then
        NMolNum = NMolNum + 1
      end if
    end if
  end do
  !
  allocate( r_Press      (NLev) )
  allocate( r_Height     (NLev) )
  allocate( r_Temp       (NLev) )
  allocate( m_MolNum     (NMolNum) )
  allocate( rm_VMRDryBase(NLev, NMolNum) )
  allocate( rm_VMR       (NLev, NMolNum) )
  !
  call ni3_get_var( NcID, PressName, r_Press  )
  call ni3_get_var( NcID, 'Height' , r_Height )
  call ni3_get_var( NcID, TempName , r_Temp   )
  !
  do m = 1, NMol
    m_MolNum     (m)   = MolNum(m)
    rm_VMRDryBase(:,m) = VMR(m)
  end do
  !
  iMolNum = NMol
  do m = 1, NHITRANMolNum
    VarName = 'VMR' // trim( m_HITRANMolName(m) )
    FlagVarExist = ni3_chk_var( NcID, VarName )
    if ( FlagVarExist ) then
      FlagVarSpecified = .false.
      do n = 1, NMol
        if ( m == MolNum(n) ) then
          FlagVarSpecified = .true.
          exit
        end if
      end do
      if ( .not. FlagVarSpecified ) then
        iMolNum = iMolNum + 1
        m_MolNum(iMolNum) = m
        call ni3_get_var( NcID, VarName , rm_VMRDryBase(:,iMolNum) )
      end if
    end if
  end do
  !
  call ni3_close( NcID )
  !
  allocate( r_VMRDryBaseH2O(NLev) )
  allocate( r_VMRH2O       (NLev) )
  r_VMRDryBaseH2O = 0.0d0
  do m = 1, NMolNum
    if ( m_MolNum(m) == 1 ) then
      r_VMRDryBaseH2O = rm_VMRDryBase(:,m)
      exit
    end if
  end do
  !
  ! r = N / ( N_t - N_h2o )
  ! q = N / N_t
  !
  ! N = r * ( N_t - N_h2o )
  !
  ! q = N / N_t
  !   = r * ( N_t - N_h2o ) / N_t
  !   = r * ( 1 - N_h2o / N_t )
  !   = r * ( 1 - q_h2o )
  !
  ! r_h2o = N_h2o / ( N_t - N_h2o )
  ! r_h2o * ( N_t - N_h2o ) = N_h2o
  ! r_h2o * N_t - r_h2o * N_h2o = N_h2o
  ! r_h2o * N_t = N_h2o + r_h2o * N_h2o
  ! r_h2o * N_t = N_h2o * ( 1 + r_h2o )
  ! r_h2o = N_h2o / N_t * ( 1 + r_h2o )
  ! r_h2o = q_h2o * ( 1 + r_h2o )
  ! q_h2o = r_h2o / ( 1 + r_h2o )
  !
  r_VMRH2O = r_VMRDryBaseH2O / ( 1.0d0 + r_VMRDryBaseH2O )
  do m = 1, NMolNum
    rm_VMR(:,m) = rm_VMRDryBase(:,m) * ( 1.0d0 - r_VMRH2O )
  end do
  !
  deallocate( r_VMRDryBaseH2O )
  deallocate( r_VMRH2O        )
  !
  deallocate( rm_VMRDryBase   )


  ! sort
  allocate( r_VMRTmp(NLev) )
  do m = 1, NMolNum
    do n = 1, NMolNum - 1 - m + 1
      if ( m_MolNum(n) > m_MolNum(n+1) ) then
        MolNumTmp     = m_MolNum(n  )
        m_MolNum(n  ) = m_MolNum(n+1)
        m_MolNum(n+1) = MolNumTmp
        !
        r_VMRTmp      = rm_VMR(:,n  )
        rm_VMR(:,n  ) = rm_VMR(:,n+1)
        rm_VMR(:,n+1) = r_VMRTmp
      end if
    end do
  end do
  deallocate( r_VMRTmp )
  write( 6, * ) 'Sorted molecular number: ', m_MolNum

  Ps = r_Press(1)
  Ts = r_Temp (1)


  ! Set particles
  !
  allocate( z_Press(1:NLev-1) )
  do k = 1, NLev-1
    z_Press(k) = ( r_Press(k) + r_Press(k+1) ) / 2.0d0
  end do
  allocate( z_Height(1:NLev-1) )
  do k = 1, NLev-1
    z_Height(k) = ( r_Height(k) + r_Height(k+1) ) / 2.0d0
  end do

  ! temporarily set
!!$  NPtcl              = 0
  NPtcl              = 1
  !
  allocate( a_PtclNum (NPtcl) )
  allocate( a_PtclName(NPtcl) )
  allocate( a_PtclDens(NPtcl) )
  allocate( za_PtclEffRad(NLev-1,NPtcl) )
  allocate( za_PtclMMR   (NLev-1,NPtcl) )
  !
  do n = 1, NPtcl
    a_PtclNum(n) = n
  end do
  a_PtclName         = ""
  a_PtclDens         = 0.0d0
  za_PtclEffRad      = 0.0d0
  za_PtclMMR         = 0.0d0

  ! cloud (Halthore et al., 2005)
  a_PtclName(1)      = "cloud"
  a_PtclDens(1)      = 1.0d3
  za_PtclEffRad(:,1) = 7.0d-6
  m = 1
  do k = 1, NLev-1
!    if ( ( 805.0d2 <= z_Press(k) ) .and. ( z_Press(k) <= 904.0d2 ) ) then
    if ( ( 1.0d3 <= z_Height(k) ) .and. ( z_Height(k) <= 2.0d3 ) ) then
!!$      za_PtclMMR(k,m) = &
!!$        &   146.0d0 * 1.0d6                  & ! number density (particles m-3)
!!$        & * 4.0d0 / 3.0d0 * 3.141592d0 * za_PtclEffRad(k,m)**3 & ! (m3 particle-1)
!!$        & * a_PtclDens(m)                                      & ! (kg m-3)
!!$        & * ( r_Height(k+1) - r_Height(k) )                    & ! (m)
!!$        & / ( ( r_Press(k) - r_Press(k+1) ) / 9.8d0 )
      ! OD = QExt * pi * r**2 * N
      !    = QExt * pi * r**2 * M / ( 4/3 * pi * r**3 * rho )
      !    = QExt * pi * r**2 / ( 4/3 * pi * r**3 * rho ) * M
      ! M  = OD / ( QExt * pi * r**2 ) * ( 4/3 * pi * r**3 * rho )
      za_PtclMMR(k,m) = &
        &   60.0d0                                            & ! optical depth
        & / ( 2.09d0 * 3.141592d0 * za_PtclEffRad(k,m)**2 )    &
        & * 4.0d0 / 3.0d0 * 3.141592d0 * za_PtclEffRad(k,m)**3 & ! (m3 particle-1)
        & * a_PtclDens(m)                                      & ! (kg m-3)
        & / ( ( r_Press(k) - r_Press(k+1) ) / 9.8d0 )
    end if
  end do


  call IOProfileOutput( &
    & OutNcFn, &
    & NLev, NMolNum, r_Press, m_MolNum, &
    & r_Temp, rm_VMR, Ps, Ts, &
    & z_Press, &
    & NChar, NPtcl, a_PtclNum, a_PtclName, &
    & a_PtclDens, za_PtclEffRad, za_PtclMMR &
    & )

end program main

!------------------------------------------------------------------------------
