  module cntnm_module

    use vtype_module

    implicit none


    integer(i4b)             , save :: s_km, s_nwn
    real(dp)    , allocatable, save :: s_wn( : ), &
         s_od_tot( :, : ), &
         s_od_h2o( :, : ), &
         s_od_co2( :, : ), &
         s_od_o3 ( :, : ), &
         s_od_o2 ( :, : ), &
         s_od_n2 ( :, : )

    private

    public :: cntnm_finalize, cntnm_init, cntnm_get_od


    !**************************************************************************

  contains

    !**************************************************************************

    subroutine cntnm_finalize


      !
      ! local variables
      !


      deallocate( s_wn, s_od_tot, s_od_h2o, s_od_co2, s_od_o3, s_od_o2, s_od_n2 )


    end subroutine cntnm_finalize

    !**************************************************************************
!!$
!!$    subroutine cntnm_init_old( km, gp, gt, gcnd_tot, &
!!$         gvmr_h2o, gvmr_co2, gvmr_o2, gvmr_n2 )
!!$
!!$      integer(i4b), intent(in ) :: km
!!$      real(dp)    , intent(in ) :: gp( km ), gt( km ), gcnd_tot( km ), &
!!$           gvmr_h2o( km ), gvmr_co2( km ), gvmr_o2( km ), gvmr_n2( km )
!!$
!!$
!!$      !
!!$      ! local variables
!!$      !
!!$      real(dp)     :: wns, wne, dwn
!!$      integer(i4b) :: m
!!$
!!$
!!$      s_km  = km
!!$      s_nwn = 5001
!!$      allocate( s_wn( s_nwn ), s_od( s_nwn, s_km ) )
!!$
!!$
!!$      wns =    0.0d2
!!$      wne = 5000.0d2
!!$      dwn = ( wne - wns ) / dble( s_nwn - 1 )
!!$
!!$
!!$      do m = 1, s_nwn
!!$         s_wn( m ) = dwn * dble( m - 1 ) + wns
!!$      end do
!!$
!!$
!!$      call cntnm_interface( km, gp, gt, gcnd_tot, &
!!$         gvmr_h2o, gvmr_co2, gvmr_o2, gvmr_n2, s_nwn, s_wn, s_od, 1, s_nwn )
!!$
!!$
!!$    end subroutine cntnm_init_old
!!$
    !**************************************************************************

    subroutine cntnm_init( km, gph, gp, gt, gmmmass, &
         gvmr_h2o, gvmr_co2, gvmr_o3, gvmr_o2, gvmr_n2 )

      use const_module

      integer(i4b), intent(in ) :: km
      real(dp)    , intent(in ) :: gph( km+1 ), gp( km ), gt( km ), gmmmass( km ), &
           gvmr_h2o( km ), gvmr_co2( km ), gvmr_o3( km ), gvmr_o2( km ), gvmr_n2( km )


      !
      ! local variables
      !
      real(dp)     :: gcnd_tot( km )
      real(dp)     :: gzeros  ( km, 4 )
      real(dp)     :: wns, wne, dwn
      integer(i4b) :: k, m


      s_km  = km
      s_nwn = 5001
      allocate( s_wn( s_nwn ), &
           s_od_tot( s_nwn, s_km ), &
           s_od_h2o( s_nwn, s_km ), &
           s_od_co2( s_nwn, s_km ), &
           s_od_o3 ( s_nwn, s_km ), &
           s_od_o2 ( s_nwn, s_km ), &
           s_od_n2 ( s_nwn, s_km ) )


      wns =    0.0d2
      wne = 5000.0d2
      dwn = ( wne - wns ) / dble( s_nwn - 1 )


      do m = 1, s_nwn
         s_wn( m ) = dwn * dble( m - 1 ) + wns
      end do


      do k = 1, km
         gcnd_tot( k ) = ( gph( k+1 ) - gph( k ) ) / ( grav * gmmmass( k ) )
      end do


      gzeros( :, : ) = 0.0d0


!!$      ! H2O
!!$      call cntnm_interface( km, gp, gt, gcnd_tot, &
!!$           gvmr_h2o   , gzeros(:,1), gzeros(:,2), gzeros(:,3), gzeros(:,4), &
!!$           s_nwn, s_wn, s_od_h2o, 1, s_nwn )
!!$      ! CO2
!!$      call cntnm_interface( km, gp, gt, gcnd_tot, &
!!$           gzeros(:,1), gvmr_co2   , gzeros(:,2), gzeros(:,3), gzeros(:,4), &
!!$           s_nwn, s_wn, s_od_co2, 1, s_nwn )
!!$      ! O3
!!$      call cntnm_interface( km, gp, gt, gcnd_tot, &
!!$           gzeros(:,1), gzeros(:,2), gvmr_o3    , gzeros(:,3), gzeros(:,4), &
!!$           s_nwn, s_wn, s_od_o3 , 1, s_nwn )
!!$      ! O2
!!$      call cntnm_interface( km, gp, gt, gcnd_tot, &
!!$           gzeros(:,1), gzeros(:,2), gzeros(:,3), gvmr_o2    , gzeros(:,4), &
!!$           s_nwn, s_wn, s_od_o2 , 1, s_nwn )
!!$      ! N2
!!$      call cntnm_interface( km, gp, gt, gcnd_tot, &
!!$           gzeros(:,1), gzeros(:,2), gzeros(:,3), gzeros(:,4), gvmr_n2    , &
!!$           s_nwn, s_wn, s_od_n2 , 1, s_nwn )

      call cntnm_interface( km, gp, gt, gcnd_tot, &
           gvmr_h2o   , gvmr_co2   , gvmr_o3    , gvmr_o2    , gvmr_n2    , &
           s_nwn, s_wn, s_od_tot, 1, s_nwn )


    end subroutine cntnm_init

    !**************************************************************************

    subroutine cntnm_get_od( km, nwnsl, wn, od, iwnsls, iwnsle )

      integer(i4b), intent(in ) :: km, nwnsl
      real(dp)    , intent(in ) :: wn( nwnsl )
      real(dp)    , intent(out) :: od( nwnsl, km )
      integer(i4b), intent(in ) :: iwnsls, iwnsle


      !
      ! local variables
      !
      real(dp)     :: t_od( s_nwn, s_km )
      integer(i4b) :: iwnsl, ii( iwnsls:iwnsle )
      integer(i4b) :: i, k


      do iwnsl = iwnsls, iwnsle
         ii( iwnsl ) = 1
      end do
      do i = 1, s_nwn-1
         do iwnsl = iwnsls, iwnsle
            if( s_wn( i ) .le. wn( iwnsl ) ) ii( iwnsl ) = i
         end do
      end do

      t_od( :, : ) = s_od_tot( :, : )
!!$      t_od( :, : ) = s_od_h2o( :, : ) + s_od_co2( :, : ) + s_od_o3( :, : ) &
!!$           + s_od_o2( :, : ) + s_od_n2( :, : )

      do k = 1, km
         do iwnsl = iwnsls, iwnsle
            od( iwnsl, k ) &
                 = ( t_od( ii( iwnsl ) + 1, k ) - t_od( ii( iwnsl ), k ) ) &
                 / ( s_wn( ii( iwnsl ) + 1 )    - s_wn( ii( iwnsl ) )    ) &
                 * ( wn  ( iwnsl )              - s_wn( ii( iwnsl ) )    ) &
                 + t_od( ii( iwnsl ), k )
         end do
      end do


    end subroutine cntnm_get_od

    !**************************************************************************

  end module cntnm_module

  !****************************************************************************

    subroutine cntnm_interface( km, gp, gt, gcnd_tot, &
         gvmr_h2o, gvmr_co2, gvmr_o3, gvmr_o2, gvmr_n2, &
         nwnsl, wn, od, iwnsls, iwnsle )

      use vtype_module

      implicit none

      integer(i4b), intent(in ) :: km
      real(dp)    , intent(in ) :: gp( km ), gt( km ), gcnd_tot( km ), &
           gvmr_h2o( km ), gvmr_co2( km ), gvmr_o3( km ), &
           gvmr_o2( km ), gvmr_n2( km )
      integer(i4b), intent(in ) :: nwnsl
      real(dp)    , intent(in ) :: wn( nwnsl )
      real(dp)    , intent(out) :: od( nwnsl, km )
      integer(i4b), intent(in ) :: iwnsls, iwnsle


      integer(i4b), parameter   :: nn = 5050
      real(dp)                  :: absrb( nn )

      real(dp)                  :: wns, wne, dwn

      real(dp)                  :: press, temp, cnd_tot
      real(dp)                  :: vmr_h2o, vmr_co2, vmr_o3, vmr_o2, vmr_n2, vmr_ar

      integer(i4b)              :: k, m


      if( nwnsl .gt. nn ) then
         write( 6, * ) 'nwnsl is too large'
         write( 6, * ) nwnsl, nn
         stop
      end if

      if( iwnsle - iwnsls .le. 1 ) then
         write( 6, * ) 'iwnsle - iwnsls is too small'
         write( 6, * ) iwnsls, iwnsle
         stop
      end if


      wns = wn( iwnsls )
      wne = wn( iwnsle )
      dwn = wn( iwnsls+1 ) - wn( iwnsls )


      wns = wns * 1.0d-2
      wne = wne * 1.0d-2
      dwn = dwn * 1.0d-2



      do k = 1, km

         press   = gp( k )
         temp    = gt( k )

         vmr_h2o = gvmr_h2o( k )
         vmr_co2 = gvmr_co2( k )
         vmr_o3  = gvmr_o3 ( k )
         vmr_o2  = gvmr_o2 ( k )
         vmr_n2  = gvmr_n2 ( k )


         vmr_ar  = 0.009d0


!!$         cnd_tot = press / ( boltz * temp ) * 1.0d-2 * 1.0d6
         cnd_tot = gcnd_tot( k )


         call drcntnm( press, temp, cnd_tot, &
              vmr_h2o, vmr_co2, vmr_o3, vmr_o2, vmr_n2, vmr_ar, &
              wns, wne, dwn, nn, absrb )

         do m = iwnsls, iwnsle
            od( m, k ) = absrb( m - iwnsls + 1 )
         end do

      end do


    end subroutine cntnm_interface

    !**************************************************************************

    subroutine drcntnm( press, temp, cnd_tot, &
         vmr_h2o, vmr_co2, vmr_o3, vmr_o2, vmr_n2, vmr_ar, &
         a_v1abs, a_v2abs, a_dvabs, nwn, a_absrb )

      use vtype_module

      IMPLICIT REAL*8           (V)

      real(dp)    , intent(in ) :: press, temp, cnd_tot
      real(dp)    , intent(in ) :: vmr_h2o, vmr_co2, vmr_o3, vmr_o2, vmr_n2, vmr_ar
      real(dp)    , intent(in ) :: a_v1abs, a_v2abs, a_dvabs
      integer(i4b), intent(in ) :: nwn
      real(dp)    , intent(out) :: a_absrb( nwn )


      COMMON /ABSORB/ V1ABS,V2ABS,DVABS,NPTABS,ABSRB(5050)

      COMMON /CVRCNT/ HNAMCNT,HVRCNT

      COMMON /FILHDR/ XID(10),SECANT,PAVE,TAVE,HMOLID(60),XALTZ(4),     &
                      WK(60),PZL,PZU,TZL,TZU,WBROAD,DV ,V1 ,V2 ,TBOUND, &
                      EMISIV,FSCDID(17),NMOL,LAYER ,YI1,YID(10),LSTWDF

      Common /share/ HOLN2

      COMMON /CONSTS/ PI,PLANCK,BOLTZ,CLIGHT,AVOGAD,ALOSMT,GASCON, &
                      RADCN1,RADCN2 
      COMMON /LAMCHN/ ONEPL,ONEMI,EXPMIN,ARGMIN
      COMMON /XCONT/  V1C,V2C,DVC,NPTC,C(6000) 

      !********************************************
      COMMON /cnth2o/ V1h,V2h,DVh,NPTh,Ch(5050),csh2o(5050),cfh2o(5050)
      !********************************************

      COMMON /IFIL/ IRD,IPRcnt,IPU,NOPR,NFHDRF,NPHDRF,NFHDRL,NPHDRL, &
                    NLNGTH,KFILE,KPANEL,LINFIL,NFILE,IAFIL,IEXFIL,   &
                    NLTEFL,LNFIL4,LNGTH4

      common /cntscl/ XSELF,XFRGN,XCO2C,XO3CN,XO2CN,XN2CN,XRAYL

      !c------------------------------------
      !c for analytic derivative calculation
      !c note: ipts  = same dimension as ABSRB
      !c       ipts2 = same dimension as C
      parameter (ipts=5050,ipts2=6000)
      common /CDERIV/ icflg,iuf,v1absc,v2absc,dvabsc,nptabsc, &
          dqh2oC(ipts),dTh2oC(ipts),dUh2o,                    &
          dqco2C(ipts),dTco2C(ipts),                          &
          dqo3C(ipts),dTo3C(ipts),                            &
          dqo2C(ipts),dTo2C(ipts),                            &
          dqn2C(ipts),dTn2C(ipts)

      real ctmp(ipts2),cself(ipts),cforeign(ipts),ch2o(ipts2)
      real ctmp2(ipts2),ctmp3(ipts2),ctmp4(ipts),ctmp5(ipts)

      data icflg/-999/
      !c------------------------------------

      dimension xcnt(7)

      equivalence (xself,xcnt(1))

      CHARACTER*18 HNAMCNT,HVRCNT

      CHARACTER*8      XID,       HMOLID,      YID 
      REAL*8               SECANT,       XALTZ

      character*8 holn2

      DATA XLOSMT/2.68675E+19/

      RADCN1=2.*PLANCK*CLIGHT*CLIGHT*1.E-07
      RADCN2=PLANCK*CLIGHT/BOLTZ

      do i=1,7
         xcnt(i)=1.
      end do

      do i=1,5050
         absrb(i)=0.
      end do

      do i=1,60
         wk(i)=0.
      end do

      ird = 55
      ipr = 66
      ipu = 7

      ipr = 6
      ipu = 6

!!$      OPEN (ipr,FILE='CNTNM.OPTDPT')
!!$      OPEN (ipu,FILE='WATER.COEF')

!!$      print *
!!$      print *, '  This version is limited to 5000 values  '
      !C
      !C   THIS PROGRAM CALCULATES THE CONTINUUM OPTICAL DEPTH
      !C         FOR AN HOMOGENEOUS LAYER
      !C
      !C   THE FOLLOWING QUANTITIES MUST BE SPECIFIED: 
      !C
      !C          PRESSURE                   PAVE (MB)
      !C
      !C          TEMPERATURE                TAVE ( K)
      !C
      !C          COLUMN AMOUNT
      !C            NITROGEN                 WN2    (MOLEC/CM**2)
      !C            OXYGEN                   WK(7)  (MOLEC/CM**2)
      !C            CARBON DIOXIDE           WK(2)  (MOLEC/CM**2)
      !C            WATER VAPOR              WK(1)  (MOLEC/CM**2)
      !C
      !C          NUMBER OF MOLECULES        NMOL
      !C
      !C          BEGINNING WAVENUMBER       V1ABS (CM-1)
      !C
      !C          ENDING WAVENUMBER          V2ABS (CM-1)
      !C
      !C          SAMPLING INTERVAL          DVABS (CM-1)
      !C
      !C          NUMBER OF VALUES           NPTABS
      !C
      !C
      !C   THE RESULTS ARE IN ARRAY ABSORB
      !C
      !C   NOTE THAT FOR AN ATMOSPHERIC LAYER: 
      !C
      !C            WTOT   = XLOSMT * (PAVE/1013) * (273/TAVE) * (PATH LENGTH)
      !C
      !C            WBROAD = the column amount for all species not explicitly provided
      !C
      !C            WK(M)  = (VOLUME MIXING RATIO) * (COLUMN OF DRY AIR)
      !C
      !C
      iprcnt = ipr
                   CALL PRCNTM 
      iprcnt = ipu
                   CALL PRCNTM

                   !C   THE FOLLOWING IS AN EXAMPLE FOR A ONE CM PATH (SEE CNTNM.OPTDPT FOR RESULTS)


!!$      PAVE = 1013.
!!$      TAVE =  296.
!!$
!!$      VMRH2O = 0.01
!!$
!!$      xlength = 1.



      PAVE = press * 1.0d-2
      TAVE = temp

      VMRH2O = vmr_h2o

      xlength = 1.



!!$      print *
!!$      print *,' *** For this program, vmr_h2o is taken ', &
!!$           'with respect to the total column ***'
!!$
!!$      print *
!!$      print *,' read: pressure (mb)  if negative use default values'
!!$      read *, press_rd
!!$      
!!$      if (press_rd .gt. 0.) then
!!$         pave = press_rd
!!$         print *,' read:   temperature (K)'
!!$         read *, tave
!!$         print *,' read:   path length (cm)'
!!$         read *, xlength
!!$         print *,' read:   vmr h2o '
!!$         read *, vmrh2o
!!$      endif
!!$      print *, 'Pressure (mb), Temperature (K), Path Length (cm),    VMR H2O'
!!$
!!$      print 911, pave,tave,xlength,vmrh2o
!!$ 911  format(1x,f13.6,f17.4,f18.4,f12.8)
!!$      !C
!!$      !c     It may be preferable to specifiy the water column directly!
!!$      !c
!!$      WTOT = XLOSMT*(PAVE/1013.)*(273./TAVE)* xlength
!!$      !C
!!$      W_dry = WTOT * (1.-VMRH2O)
!!$      !c
!!$      !c     ww is column of dry air;  vol. mix. ratios are based on dry air
!!$      !c
!!$      !c argon:
!!$      WA     = 0.009     * W_dry
!!$      !c nitrogen:
!!$      WN2    = 0.78      * W_dry
!!$      !c oxygen:
!!$      WK(7)  = 0.21      * W_dry
!!$
!!$      !c carbon dioxide:
!!$      WK(2)  = 345.E-06  * W_dry
!!$
!!$      !c water vapor:
!!$      if (abs(vmrh2o-1.) .lt. 1.-e-05) then
!!$         wk(1) = wtot
!!$      else
!!$         WK(1) = VMRH2O * W_dry
!!$      endif






      !C
      !c     It may be preferable to specifiy the water column directly!
      !c
!!$      WTOT = XLOSMT*(PAVE/1013.)*(273./TAVE)* xlength
      WTOT = cnd_tot * 1.0d-4
      !C
      W_dry = WTOT * (1.-VMR_H2O)
      !c
      !c     ww is column of dry air;  vol. mix. ratios are based on dry air
      !c
      !c argon:
      WA     = vmr_ar    * W_dry
      !c nitrogen:
      WN2    = vmr_n2    * W_dry
      !c oxygen:
      WK(7)  = vmr_o2    * W_dry
      !c carbon dioxide:
      WK(2)  = vmr_co2   * W_dry
      !c water vapor:
      WK(1)  = vmr_h2o   * W_dry


      !c ozone:
      WK(3)  = vmr_o3   * W_dry






      WBROAD=WN2+WA

      NMOL = 7

!!$      V1ABS =    0.
!!$      V2ABS = 5000. 
!!$
!!$      DVABS =    10.


      V1ABS = a_v1abs
      V2ABS = a_v2abs
      DVABS = a_dvabs


      !c ..........................................................
      !c      write (*,*) '  v1abs,  v2abs,  dvabs  '
      !c      read  (*,*)    v1abs,  v2abs,  dvabs
      !c ..........................................................

      NPTABS =  1. + (v2abs-v1abs)/dvabs

      do i=1,nptabs
         absrb(i) =0.
      end do


!!$      WRITE (IPR,970) PAVE,TAVE
!!$      WRITE (IPR,975) (HMOLID(I),I=1,7),HOLN2
!!$      WRITE (IPR,980) (WK(M),M=1,7),WBROAD
!!$
!!$      WRITE (IPu,970) PAVE,TAVE
!!$      WRITE (IPu,975) (HMOLID(I),I=1,7),HOLN2
!!$      WRITE (IPu,980) (WK(M),M=1,7),WBROAD
!!$
!!$  970 FORMAT (/,29x, 'P(MB)',7X,'T(K)', //,23x,0P,F12.3,F9.2)
!!$  975 FORMAT (/, 9X,'MOLECULAR AMOUNTS (MOL/CM**2) BY LAYER ',//, &
!!$               8(1X,A6,3X))
!!$  980 FORMAT (/,1P,8E10.3,//)

      jrad=1

      v1 = v1abs
      v2 = v2abs

      CALL CONTNM(JRAD)

!!$      DO I=1,NPTABS
!!$         VI=V1ABS+FLOAT(I-1)*DVABS
!!$         WRITE (ipr, 910) VI, ABSRB(I) 
!!$      end DO
!!$910   FORMAT(F10.3,1P,E12.3)
!!$
!!$      WRITE (7,920) tave
!!$  920 FORMAT(//,' self and foreign water vapor continuum coefficients ',/, &
!!$             'for  ',f8.2,'K - ',   //,                                    &
!!$             ' the self-continuum scales as ( Rself/Ro ) ',/,              &
!!$             ' the foreign continuum scales as ( (Rtot-Rself)/Ro ) ',/,    &
!!$             ' where R is the density rho [ R = (P/Po)*(To/T) ]. ',//,     &
!!$         10x,'     without radiation field:  ',                            &
!!$         10x,'       with radiation field:   ',      /,                    &
!!$         10x,'      self         foreign     ',                            &
!!$         10x,'      self         foreign     ',      /,                    &
!!$             '    cm-1  ',                                                 &
!!$             '       1/(cm-1 molec/cm**2)    ',                            &
!!$         10x,'       1/(molec/cm**2) '        ,//)
!!$
!!$      xkt=tave/radcn2
!!$
!!$      do i=1,npth
!!$         vi=v1h+float(i-1)*dvh
!!$         if (vi.ge.v1abs .and. vi.le.v2abs) then
!!$            radfld=radfn(vi,xkt)
!!$            csh2or=csh2o(i) * radfld
!!$            cfh2or=cfh2o(i) * radfld
!!$            write (ipu,930) vi, csh2o(i), cfh2o(i), csh2or, cfh2or
!!$930         format(f10.2, 1p, 2e15.4,10x, 1p, 2e15.4)
!!$         endif
!!$      end do




      a_absrb( : ) = -1.0d0
      do i = 1, nptabs
         a_absrb( i ) = absrb( i )
      end do


    end subroutine drcntnm
