  program main

    use vtype_module
    use ni3_module
    use netcdf

    implicit none

    real(dp), parameter   :: MeanMolWeight = 28.966d-3

    real(dp), parameter   :: VMRCO2 = 400.0d-6

    character(len=extstr)            :: infn


    integer(i4b)                     :: km

    real(dp)    , allocatable        :: a_Height( : )
    real(dp)    , allocatable        :: a_Press ( : )
    real(dp)    , allocatable        :: a_Temp  ( : )
    real(dp)    , allocatable        :: a_VMRH2O( : )
    real(dp)    , allocatable        :: a_VMRCO2( : )
    real(dp)    , allocatable        :: a_VMRO3 ( : )

    character(len=extstr)            :: ncfn
    integer(i4b)                     :: ncid
    character(len=extstr)            :: mode, title, com, &
         name, stdname, longname, units
    character(len=extstr)            :: src
    character(len=extstr)            :: ref
    integer(i4b)         , parameter :: ndims = 1
    character(len=extstr)            :: dimname( ndims )
!!$    real(dp)                         :: fillvalue


    title   = 'Tropical Atmosphere'
    src     = "Table 5 of Ellingson et al. (1991)"
    infn    = 'data/Pro1D_Tropics.nc'
    ncfn    = "out/Pro1D_Tropics.nc"

!!$    title   = 'Mid-Latitude Summer Atmosphere'
!!$    src     = "Table 6 of Ellingson et al. (1991)"
!!$    infn    = 'data/Pro1D_MidLatSummer.nc'
!!$    ncfn    = "out/Pro1D_MidLatSummer.nc"
!!$
!!$    title   = 'Mid-Latitude Winter Atmosphere'
!!$    src     = "Table 7 of Ellingson et al. (1991)"
!!$    infn    = 'data/Pro1D_MidLatWinter.nc'
!!$    ncfn    = "out/Pro1D_MidLatWinter.nc"
!!$
!!$    title   = 'Subarctic Summer Atmosphere'
!!$    src     = "Table 8 of Ellingson et al. (1991)"
!!$    infn    = 'data/Pro1D_SubArcSummer.nc'
!!$    ncfn    = "out/Pro1D_SubArcSummer.nc"
!!$
!!$    title   = 'Subarctic Winter Atmosphere'
!!$    src     = "Table 9 of Ellingson et al. (1991)"
!!$    infn    = 'data/Pro1D_SubArcWinter.nc'
!!$    ncfn    = "out/Pro1D_SubArcWinter.nc"



    !
    ! Input
    !
    Mode = "read"
    call ni3_open( InFN, Mode, NcID )
    call ni3_inq_dimlen( NcID, "plev", km )

    allocate( a_Height( km ) )
    allocate( a_Press ( km ) )
    allocate( a_Temp  ( km ) )
    allocate( a_VMRH2O( km ) )
    allocate( a_VMRCO2( km ) )
    allocate( a_VMRO3 ( km ) )

    call ni3_get_var( NcID, "plev"  , a_Press  )
    call ni3_get_var( NcID, "Height", a_Height )
    call ni3_get_var( NcID, "Temp"  , a_Temp   )
    call ni3_get_var( NcID, "QH2O"  , a_VMRH2O )
    call ni3_get_var( NcID, "QO3"   , a_VMRO3  )

    call ni3_close( NcID )

    a_VMRH2O    = a_VMRH2O * MeanMolWeight / 0.0180152636021376d0
    a_VMRO3     = a_VMRO3  * MeanMolWeight / 0.0479970946907997d0

    a_VMRCO2    = VMRCO2


    !
    ! Output to a NetCDF file
    !
    mode = "new"
    call ni3_open( ncfn, mode, ncid )

    title = "Data for " // trim( title )
    com   = "Volume mixing ratios are calculated with mean atmospheric molecular weight of 28.966e-3 kg mol-1."
    ref   = "Ellingson, R. G., J. Ellis, and S. Fels (1991), The intercomparison of radiation codes used in climate models: Long wave results, J. Geophys. Res., 96, 8929. http://onlinelibrary.wiley.com/doi/10.1029/90JD01450/pdf"
    call ni3_set_ga( ncid, title = title, com = com, src = src, ref = ref )


    name     = "plev"
    stdname  = "plev"
    longname = 'pressure'
    units    = "Pa"
    call ni3_set_dim( ncid, name, NF90_REAL, a_Press, &
      stdname = stdname, longname = longname, units  = units )
    call ni3_put_att( ncid, name, 'positive', 'down' )

    dimname( 1 ) = "plev"

    name         = "Height"
    stdname      = "height"
    longname     = stdname
    units        = "m"
    call ni3_def_var( ncid, name, NF90_REAL, ndims, dimname, &
         stdname = stdname, longname = longname, units = units )
    call ni3_put_var( ncid, name, a_Height )

    name         = "Temp"
    stdname      = "air_temperature"
    longname     = stdname
    units        = "K"
    call ni3_def_var( ncid, name, NF90_REAL, ndims, dimname, &
         stdname = stdname, longname = longname, units = units )
    call ni3_put_var( ncid, name, a_Temp )

    name         = "VMRH2O"
    stdname      = "water_vapor"
    longname     = "Volume mixing ratio of H2O vapor"
    units        = "1"
    call ni3_def_var( ncid, name, NF90_REAL, ndims, dimname, &
         stdname = stdname, longname = longname, units = units )
    call ni3_put_var( ncid, name, a_VMRH2O )

    name         = "VMRCO2"
    stdname      = "carbon_dioxide"
    longname     = "Volume mixing ratio of carbon dioxide"
    units        = "1"
    call ni3_def_var( ncid, name, NF90_REAL, ndims, dimname, &
         stdname = stdname, longname = longname, units = units )
    call ni3_put_var( ncid, name, a_VMRCO2 )

    name         = "VMRO3"
    stdname      = "ozone"
    longname     = "Volume mixing ratio of ozone"
    units        = "1"
    call ni3_def_var( ncid, name, NF90_REAL, ndims, dimname, &
         stdname = stdname, longname = longname, units = units )
    call ni3_put_var( ncid, name, a_VMRO3 )

    call ni3_close( ncid )


  end program main
