c******************************************************************************
c     real*8 function voigt
c     calculate line shape factor of voigt type
c******************************************************************************
c     pi            :
c     lorwid        : lorentz halfwidth (m^-1)
c     dopwid        : doppler halfwidth (m^-1)
c     dv            : dv=(v-v0) : v  : wave number
c                   :           : v0 : wave number at line center
c                   : this value will be casted absolute value
c******************************************************************************

C***********************************************************************
      real*8 FUNCTION VOIGT (pi,lorwid,dopwid,dv)
C***********************************************************************
C****
C****  ROUTINE COMPUTES THE VOIGT FUNCTION: Y/PI*INTEGRAL FROM  ***
C****   - TO + INFINITY OF EXP(-T*T)/(Y*Y+(X-T)*(X-T)) DT       ***
C****
      real*8 pi
      real*8 lorwid
      real*8 dopwid
      real*8 dv

      REAL*8 B(22),RI(15),XN(15),YN(15)
      REAL*8 D0(25),D1(25),D2(25),D3(25),D4(25)
      REAL*8 HN(25),H,XX(3),HH(3),NBY2(19),C(21)
      LOGICAL TRU
      DATA B(1)/0.0/,B(2)/0.7093682E-7/
      DATA XN/10.0,9.0,2*8.0,7.0,6.0,5.0,4.0,7*3.0/
      DATA YN/3*0.6,0.5,2*0.4,4*0.3,1.0,0.9,0.8,2*0.7/
      DATA H/0.201/
      DATA XX/0.5246476,1.65068,0.7071068/
      DATA HH/0.2562121,0.2588268E-1,0.2820948/
      DATA NBY2/9.5,9.0,8.5,8.0,7.5,7.0,6.5,6.0,5.5,5.0,4.5,4.0,3.5,3.0,
     &  2.5,2.0,1.5,1.0,0.5/
      DATA C / 0.7093682E-7,-0.2518434E-6, 0.8566874E-6,-0.2787638E-5,
     &         0.8668740E-5,-0.2565551E-4, 0.7228775E-4,-0.1933631E-3,
     &         0.4899520E-3,-0.1173267E-2, 0.2648762E-2,-0.5623190E-2,
     &         0.1119681E-1,-0.2084976E-1, 0.3621573E-1,-0.5851412E-1,
     &         0.8770816E-1,-0.121664, 0.15584, -0.184, 0.2           /
      DATA TRU/.FALSE./

      real*8 x,y

      x=abs(dv)/dopwid
      y=lorwid/dopwid

      IF (TRU) GO TO 104
C****
C****  REGION I: COMPUTE DAWSON'S FUNCTION AT MESH POINTS. ***
C****
      TRU=.TRUE.
      DO 101 I=1,15
        RI(I)=-I/2.0
  101 CONTINUE
      DO 103 I=1,25
        HN(I)=H*(I-0.5)
        CO=4*HN(I)*HN(I)/25-2
        DO 102 J=2,21
          B(J+1)=CO*B(J)-B(J-1)+C(J)
  102   CONTINUE
        D0(I)=HN(I)*(B(22)-B(21))/5
        D1(I)=1-2*HN(I)*D0(I)
        D2(I)=(HN(I)*D1(I)+D0(I))/RI(2)
        D3(I)=(HN(I)*D2(I)+D1(I))/RI(3)
        D4(I)=(HN(I)*D3(I)+D2(I))/RI(4)
  103 CONTINUE
  104 CONTINUE
      IF (X.GE.5.0) GO TO 112
      IF (Y.LE.1.0) GO TO 110
      IF (X.GT.1.85*(3.6-Y)) GO TO 112
C****
C****  REGION II: CONTINUED FRACTION. COMPUTE NUMBER OF TERMS NEEDED. ***
C****
      IF (Y.LT.1.45) GO TO 107
      I=Y+Y
      GO TO 108
  107 CONTINUE
      I=11*Y
  108 CONTINUE
      J=X+X+1.85
      MAX=XN(J)*YN(I)+0.46
      MIN=MIN0(16,21-2*MAX)
C****
C****  EVALUATE CONTINUED FRACTION. ***
C****
      UU=Y
      VV=X
      DO 109 J=MIN,19
        U=NBY2(J)/(UU*UU+VV*VV)
        UU=Y+U*UU
        VV=X-U*VV
  109 CONTINUE
      VOIGT=UU/(UU*UU+VV*VV)/1.772454
      voigt=1.0d0/(dopwid*sqrt(pi))*voigt
      RETURN
C****
  110 CONTINUE
      Y2=Y*Y
      IF (X+Y.GE.5.0) GO TO 113
C****
C****  REGION I: COMPUTE DAWSON'S FUNCTION AT X FROM TAYLOR SERIES. ***
C****
      N=X/H
      DX=X-HN(N+1)
      U=(((D4(N+1)*DX+D3(N+1))*DX+D2(N+1))*DX+D1(N+1))*DX+D0(N+1)
      V=1-2*X*U
C****
C****  TAYLOR SERIES EXPANSION ABOUT Y=0.0. ***
C****
      VV=EXP(Y2-X*X)*COS(2*X*Y)/1.128379-Y*V
      UU=-Y
      MAX=5+(12.5-X)*0.8*Y
      DO 111 I=2,MAX,2
        U=(X*V+U)/RI(I)
        V=(X*U+V)/RI(I+1)
        UU=-UU*Y2
        VV=VV+V*UU
  111 CONTINUE
      VOIGT=1.128379*VV
      voigt=1.0d0/(dopwid*sqrt(pi))*voigt
      RETURN
C****
  112 CONTINUE
      Y2=Y*Y
      IF (Y.LT.11-0.6875*X) GO TO 113
C****
C****  REGION IIIB: 2-POINT GAUSS-HERMITE QUADRATURE. ***
C****
      U=X-XX(3)
      V=X+XX(3)
      VOIGT=Y*(HH(3)/(Y2+U*U)+HH(3)/(Y2+V*V))
      voigt=1.0d0/(dopwid*sqrt(pi))*voigt
      RETURN
C****
C****  REGION IIIA: 4-POINT GAUSS-HERMITE QUADRATURE. ***
C****
  113 CONTINUE
      U=X-XX(1)
      V=X+XX(1)
      UU=X-XX(2)
      VV=X+XX(2)
      VOIGT=Y*(HH(1)/(Y2+U*U)+HH(1)/(Y2+V*V)
     &      +HH(2)/(Y2+UU*UU)+HH(2)/(Y2+VV*VV))
      voigt=1.0d0/(dopwid*sqrt(pi))*voigt
      RETURN
      END
