program main

  use vtype_module
  use fi_module
  use particle_ec_calc, only : ParticleECCalc

  implicit none


  character(extstr) :: CtlFN  = 'calc_optprop.conf'
  character(extstr) :: CtlFN2 = 'calc_optprop_file.conf'

  integer           :: iostat

  character(len=extstr)          :: OutputFileName

  real(dp)                           :: DelWaveNum
  real(dp)                           :: WaveNumS
  real(dp)                           :: WaveNumE

  logical            :: FlagReadQExtSSAAFFromFile

  integer, parameter :: MaxNmlArySize = 32

  integer            :: NMolNum
  integer            :: MolNum(MaxNmlArySize)

  integer            :: NPtcl
  character(stdstr)  :: PtclName   (MaxNmlArySize)
  character(extstr)  :: PtclDataFN (MaxNmlArySize)
  integer            :: PtclNRadius(MaxNmlArySize)
  real(dp)           :: PtclRadiusS(MaxNmlArySize)
  real(dp)           :: PtclRadiusE(MaxNmlArySize)

  integer          , allocatable :: m_MolNum(:)

  character(stdstr), allocatable :: a_PtclName  (:)
  character(extstr), allocatable :: a_PtclDataFN(:)

  integer          , allocatable :: a_PtclNRadius(:)
  real(dp)         , allocatable :: a_PtclRadiusS(:)
  real(dp)         , allocatable :: a_PtclRadiusE(:)

  integer(i4b)              :: FileUnit

  integer(i4b)              :: m
  integer(i4b)              :: n


  namelist /wn_nml/ &
    & DelWaveNum, WaveNumS, WaveNumE
  namelist /ptcl_nml/ &
    & FlagReadQExtSSAAFFromFile, &
    & NMolNum, MolNum, &
    & NPtcl, PtclName, PtclDataFN, &
    & PtclNRadius, PtclRadiusS, PtclRadiusE
  namelist /file_nml/ &
    & OutputFileName

  OutputFileName = "./out/ac.nc"

!!$      ! for 15 mincron band
!!$      WaveNumS =  400.0d0 * 1.0d2
!!$      WaveNumE =  900.0d0 * 1.0d2
!!$      ! for CO2 500-900 cm^-1 for validation
!!$      WaveNumS =  500.0d0 * 1.0d2
!!$      WaveNumE =  900.0d0 * 1.0d2
  ! for 15 mincron band for validation
  DelWaveNum = 1.0d-2 * 1.0d2
  WaveNumS =  550.0d0 * 1.0d2 + DelWaveNum / 2.0d0
  WaveNumE =  750.0d0 * 1.0d2 - DelWaveNum / 2.0d0

  WaveNumS =  600.0d0 * 1.0d2 + DelWaveNum / 2.0d0
  WaveNumE =  601.0d0 * 1.0d2 - DelWaveNum / 2.0d0


  FlagReadQExtSSAAFFromFile = .false.

  NMolNum   = 1
  MolNum    = -1
  MolNum(1) = 1
!!$      WaveNumS =    0.0d0 * 1.0d2
!!$      WaveNumE = 3000.0d0 * 1.0d2

  NPtcl         = 0
  PtclName  (:) = ""
  PtclDataFN(:) = ""

  PtclNRadius(:) = 0
  PtclRadiusS(:) = -1.0d100
  PtclRadiusE(:) = -1.0d100


  ! Read a namelist file

  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = wn_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in wn_nml is something wrong.'
  end if
  write( 6, nml = wn_nml )
  !
  rewind( FileUnit )
  read( FileUnit, nml = ptcl_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in ptcl_nml is something wrong.'
  end if
  write( 6, nml = ptcl_nml )
  !
  close( FileUnit )

  call fi_open( CtlFN2, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = file_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in file_nml is something wrong.'
  end if
  write( 6, nml = file_nml )
  !
  close( FileUnit )


  allocate( m_MolNum         (NMolNum  ) )
  allocate( a_PtclName   (NPtcl) )
  allocate( a_PtclDataFN (NPtcl) )
  allocate( a_PtclNRadius(NPtcl) )
  allocate( a_PtclRadiusS(NPtcl) )
  allocate( a_PtclRadiusE(NPtcl) )

  m_MolNum = MolNum(1:NMolNum)

  a_PtclName    = PtclName   (1:NPtcl)
  a_PtclDataFN  = PtclDataFN (1:NPtcl)
  a_PtclNRadius = PtclNRadius(1:NPtcl)
  a_PtclRadiusS = PtclRadiusS(1:NPtcl)
  a_PtclRadiusE = PtclRadiusE(1:NPtcl)

  do m = 1, NMolNum
    if ( m_MolNum(m) <= 0 ) then
      write( 6, * ) 'Molecular number is inappropriate: ', m_MolNum
      stop
    end if
  end do
  do n = 1, NPtcl
    if ( a_PtclNRadius(n) <= 0 ) then
      write( 6, * ) 'Number of radius has to be positive.'
      stop
    end if
  end do


  call ParticleECCalc( &
    & FlagReadQExtSSAAFFromFile, &
    & DelWaveNum, WaveNumS, WaveNumE, &
    & NMolNum, m_MolNum, &
    & NPtcl, a_PtclName, a_PtclDataFN, &
    & a_PtclNRadius, a_PtclRadiusS, a_PtclRadiusE, &
    & OutputFileName &
    & )

  deallocate( a_PtclName    )
  deallocate( a_PtclDataFN  )
  deallocate( a_PtclNRadius )
  deallocate( a_PtclRadiusS )
  deallocate( a_PtclRadiusE )

  write( 6, * ) 'Finish'

end program main

