# coding: utf-8
require "numru/ggraph"
include NumRu

sol = 88800.0

dirpath = "../out_dwn1e0"

ncfns_model = [
  [
    dirpath+"/Mischna2012_Mars_006mb_LW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_006mb_LW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_006mb_LW_Tendency.nc",
    dirpath+"/Mischna2012_Mars_006mb_SW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_006mb_SW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_006mb_SW_Tendency.nc"
  ],
  [
    dirpath+"/Mischna2012_Mars_006mb_Dry_LW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_006mb_Dry_LW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_006mb_Dry_LW_Tendency.nc",
    dirpath+"/Mischna2012_Mars_006mb_Dry_SW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_006mb_Dry_SW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_006mb_Dry_SW_Tendency.nc"
  ],
  [
    dirpath+"/Mischna2012_Mars_500mb_LW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_500mb_LW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_500mb_LW_Tendency.nc",
    dirpath+"/Mischna2012_Mars_500mb_SW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_500mb_SW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_500mb_SW_Tendency.nc"
  ],
  [
    dirpath+"/Mischna2012_Mars_500mb_Dry_LW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_500mb_Dry_LW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_500mb_Dry_LW_Tendency.nc",
    dirpath+"/Mischna2012_Mars_500mb_Dry_SW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_500mb_Dry_SW_Flux.nc", 
    dirpath+"/Mischna2012_Mars_500mb_Dry_SW_Tendency.nc"
  ]
]

vnames = ["RadDwFluxLW", "RadUwFluxLW", "DTempDtLW", "RadDwFluxSW", "RadUwFluxSW", "DTempDtSW"]
longnames = ["lw.dn.flux", "lw.up.flux", "lw.tendency", "sw.dn.flux", "sw.up.flux", "sw.tendency"]

#                      k  j  1    k  j  2
xlim = [
  [
    [  0.0,  50.0], # [0][0][0], [0][0][1]
    [205.0, 225.0], # [0][1][0], [0][1][1]
    [-80.0,   0.0], # [0][2][0], [0][2][1]
    [286.0, 293.0], # [0][3][0], [0][3][1]
    [ 57.0,  59.0], # [0][4][0], [0][4][1]
    [  0.0, 120.0]  # [0][5][0], [0][5][1]
  ],
  [
    [  0.0,  50.0], # [1][0][0], [1][0][1]
    [205.0, 225.0], # [1][1][0], [1][1][1]
    [-80.0,   0.0], # [1][2][0], [1][2][1]
    [286.0, 293.0], # [1][3][0], [1][3][1]
    [ 57.0,  59.0], # [1][4][0], [1][4][1]
    [  0.0, 120.0]  # [1][5][0], [1][5][1]
  ],
  [
    [  0.0, 200.0], # [2][0][0], [2][0][1]
    [100.0, 230.0], # [2][1][0], [2][1][1]
    [-20.0,   0.0], # [2][2][0], [2][2][1]
    [150.0, 230.0], # [2][3][0], [2][3][1]
    [ 30.0,  66.0], # [2][4][0], [2][4][1]
    [  0.0,  10.0]  # [2][5][0], [2][5][1]
  ],
  [
    [  0.0, 200.0], # [3][0][0], [3][0][1]
    [100.0, 230.0], # [3][1][0], [3][1][1]
    [-20.0,   0.0], # [3][2][0], [3][2][1]
    [150.0, 230.0], # [3][3][0], [3][3][1]
    [ 30.0,  66.0], # [3][4][0], [3][4][1]
    [  0.0,  10.0]  # [3][5][0], [3][5][1]
  ]
]

ylim = [
  [  10e2, 1e-1], # [0][0], [0][1]
  [  10e2, 1e-1], # [1][0], [1][1]
  [1000e2, 1e-1], # [2][0], [2][1]
  [1000e2, 1e-1]  # [3][0], [3][1]
]

filelistfns = [
  "filelist_6mb_water.txt", 
  "filelist_6mb_nowater.txt", 
  "filelist_500mb_water.txt",
  "filelist_500mb_nowater.txt"
]

iws = (ARGV[0] || (puts ' WORKSTATION ID (I)  ? ;'; DCL::sgpwsn; gets)).to_i
DCL.gropn(iws)


DCL.sldiv('y',3,2)
DCL.sgpset('isub', 96)   # control character of subscription: '_' --> '`'
DCL.sgpset('lfull',true)
#DCL.uzfact(0.6)
DCL.uzfact(1.2)
DCL.sglset('LCLIP', true)
DCL.sglset('LCNTL', true)

for k in 0..filelistfns.size-1 do
  filelistfn = filelistfns[k]

  y1 = ylim[k][0]
  y2 = ylim[k][1]
  for j in 0..vnames.size-1 do
    vname = vnames[j]

    x1 = xlim[k][j][0]
    x2 = xlim[k][j][1]

    GGraph.set_fig 'itr'=> 2, 'viewport'=>[0.2,0.8,0.2,0.85],
                   'window'=>[x1,x2,y1,y2]
#                   'window'=>[x1s[j],x2s[j],y1,y2]
#               'window'=>[0,1,0.0,100]

    ncfn = ncfns_model[k][j]
    gphys = GPhys::IO.open( ncfn, vname )
    gphys = gphys.copy
    gphys.long_name = longnames[j]
    if j.modulo(3) == 2 then
      gphys = gphys * sol
      gphys.units = "K (sol)-1"
    end
    if j == 1 then
      title = filelistfn[9..filelistfn.size-5]
    else
      title = ""
    end
    if j == 0 then
      legend = "this study"
    else
      legend = nil
    end
    GGraph.line gphys, true, 'exchange'=>true, 'title'=>title,
                'index'=> 5,
                'legend'=>legend, 'legend_vx'=>0.5, 'legend_vy'=>0.8

    ifile = 0
    File.foreach( filelistfn ) do |line|
      flag_draw = true

      ncfn = line.chomp
      if ncfn[0] == "#" then
        puts ncfn, " --- skipped"
        flag_draw = false
      else
        puts ncfn
      end

      if ncfn.include?("driver_data_wbm.nc") then
        if vname == "RadUwFluxSW" || vname == "RadDwFluxSW" then
          puts "  " + vname + " is skipped"
          flag_draw = false
        end
      end
      if ncfn.include?("driver_data_lblrtm.nc") then
        if vname[-2..-1] == "SW" then
          puts "  " + vname + " is skipped"
          flag_draw = false
        end
      end

      gphys = GPhys::IO.open( ncfn, vname )
      gphys = gphys.copy

      if j.modulo(3) == 2 then
        gphys = gphys * sol
        gphys.units = "K (sol)-1"
      end
      if j == 0 then
        legend = ncfn[ncfn.rindex("data_")+5..ncfn.size-4]
      else
        legend = nil
      end

      if flag_draw then
        GGraph.line gphys, false, 'exchange'=>true, 'title'=>'',
                    'index'=> 10*(ifile+2)+1,
                    'legend'=>legend, 'legend_vx'=>0.5, 'legend_vy'=>0.75-ifile*0.05
      end

      ifile += 1
    end

  end

end

DCL.grcls
