program main

  use vtype_module
  use fi_module
  use ni3_module
  use netcdf

  implicit none

  integer                        :: NcID
  character(extstr)              :: Mode
  character(extstr)              :: Name
  character(extstr)              :: StdName
  character(extstr)              :: LongName
  character(extstr)              :: Units
  integer                        :: NDims
  character(extstr), allocatable :: a_DimNames(:)

  integer                :: NPress
  integer                :: NTemp
  real(8), allocatable   :: r_Press(:)
  real(8), allocatable   :: r_Temp(:)

  integer                :: NMol
  integer, allocatable   :: m_MolNum(:)

  real(8), allocatable   :: rm_VMR(:,:)

  real(8)                :: Ps
  real(8)                :: Ts

  real(8) :: PressMin
  real(8) :: PressMax

  real(8) :: TempMin
  real(8) :: TempMax

  real(8) :: Press
  real(8) :: Temp

  character(128) :: NcFN
  character(128) :: ctlfn = 'mkprofile.conf'

  integer :: FileUnit
  integer :: iostat

  integer :: iTemp
  integer :: iVMRKeyMol1
  integer :: iVMRKeyMol2
  integer :: iVMRKeyMol3

  integer :: iVMRKeyMol1s
  integer :: iVMRKeyMol1e
  integer :: iVMRKeyMol2s
  integer :: iVMRKeyMol2e
  integer :: iVMRKeyMol3s
  integer :: iVMRKeyMol3e

  integer :: KeyMol1Num
  integer :: KeyMol2Num
  integer :: KeyMol3Num
  real(8) :: VMRMinKeyMol1
  real(8) :: VMRMaxKeyMol1
  integer :: NVMRKeyMol1
  real(8) :: VMRMinKeyMol2
  real(8) :: VMRMaxKeyMol2
  integer :: NVMRKeyMol2
  real(8) :: VMRMinKeyMol3
  real(8) :: VMRMaxKeyMol3
  integer :: NVMRKeyMol3

  integer :: IndexKeyMol1
  integer :: IndexKeyMol2
  integer :: IndexKeyMol3

  logical :: FlagWarning
  logical :: FlagPressZero = .true.
!!$  logical :: FlagPressZero = .false.

  logical :: FlagVMRZeroKeyMol1
  logical :: FlagVMRZeroKeyMol2
  logical :: FlagVMRZeroKeyMol3

  character(extstr)              :: NcFNVMR
  logical          , allocatable :: a_FlagVMRSetFromFile(:)

  logical                        :: FlagReadVMRFile
  integer                        :: NMolNumVMRFile
  integer                        :: NPressVMRFile
  integer          , allocatable :: m_MolNumVMRFile(:)
  real(dp)         , allocatable :: r_PressVMRFile (:)
  real(dp)         , allocatable :: rm_VMRFile     (:,:)

  integer, parameter :: MaxNmlArySize = 256
  integer            :: MolNum(MaxNmlArySize)
  real(8)            :: VMR(MaxNmlArySize)
  logical            :: FlagVMRSetFromFile(MaxNmlArySize)

  integer :: k
  integer :: m
  integer :: m2


  namelist /molecule_nml/ &
    & MolNum, VMR, &
    & NcFNVMR, FlagVMRSetFromFile

  namelist /range_nml/ &
    & PressMax, PressMin, NPress, &
    & TempMin, TempMax, NTemp, &
    & KeyMol1Num, VMRMinKeyMol1, VMRMaxKeyMol1, NVMRKeyMol1, &
    & FlagVMRZeroKeyMol1, &
    & KeyMol2Num, VMRMinKeyMol2, VMRMaxKeyMol2, NVMRKeyMol2, &
    & FlagVMRZeroKeyMol2, &
    & KeyMol3Num, VMRMinKeyMol3, VMRMaxKeyMol3, NVMRKeyMol3, &
    & FlagVMRZeroKeyMol3


  MolNum             = -1
  MolNum(1)          =  1
  VMR                = -1.0d0
  VMR(1)             = 1.0d-2
!!$  VMR(2)             = 369.0d-6
!!$  VMR(3)             = 1.0d-6
  !
  NcFNVMR               = ""
  FlagVMRSetFromFile(:) = .false.


  PressMax      = 1.0d5
  PressMin      = 1.0d2
  NPress        = 31

  TempMin = 150.0d0
  TempMax = 350.0d0
  NTemp   = 7


  KeyMol1Num         = 1
  VMRMinKeyMol1      = 1.0d-2
  VMRMaxKeyMol1      = 1.0d-2
  NVMRKeyMol1        = 1
  FlagVMRZeroKeyMol1 = .true.
  !
  KeyMol2Num         = 2
  VMRMinKeyMol2      = 1.0d-2
  VMRMaxKeyMol2      = 1.0d-2
  NVMRKeyMol2        = 1
  FlagVMRZeroKeyMol2 = .true.
  !
  KeyMol3Num         = 2
  VMRMinKeyMol3      = 1.0d-2
  VMRMaxKeyMol3      = 1.0d-2
  NVMRKeyMol3        = 1
  FlagVMRZeroKeyMol3 = .true.


  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = molecule_nml, iostat = iostat )
  if ( iostat /= 0 ) then
    stop 'Namelist, molecule_nml, is something wrong.'
  end if
  write( 6, nml = molecule_nml )
  !
  rewind( FileUnit )
  read( FileUnit, nml = range_nml, iostat = iostat )
  if ( iostat /= 0 ) then
    stop 'Namelist, range_nml, is something wrong.'
  end if
  write( 6, nml = range_nml )
  close( FileUnit )


  if ( FlagPressZero      ) NPress      = NPress      + 1
  if ( FlagVMRZeroKeyMol1 ) NVMRKeyMol1 = NVMRKeyMol1 + 1
  if ( FlagVMRZeroKeyMol2 ) NVMRKeyMol2 = NVMRKeyMol2 + 1
  if ( FlagVMRZeroKeyMol3 ) NVMRKeyMol3 = NVMRKeyMol3 + 1


  do m = 1, MaxNmlArySize
    if ( MolNum(m) <= 0.0d0 ) exit
  end do
  NMol = m - 1
  do m = 1, NMol
    if ( VMR(m) < 0.0d0 ) then
      write( 6, * ) 'VMR(',m,') is negative.'
      stop
    end if
  end do
  !
  allocate( m_MolNum(NMol) )
  allocate( rm_VMR(NPress, NMol) )
  allocate( a_FlagVMRSetFromFile(NMol) )

  m_MolNum = MolNum(1:NMol)
  do m = 1, NMol
    rm_VMR(:,m) = VMR(m)
  end do
  !
  a_FlagVMRSetFromFile = FlagVMRSetFromFile(1:NMol)

  do m = 1, NMol
    if ( m_MolNum(m) == KeyMol1Num ) exit
  end do
  if ( m > NMol ) then
    write( 6, * ) 'Unable to find KeyMol1=', KeyMol1Num, ' in MolNum.'
    write( 6, * ) '  MolNum = ', m_MolNum
    stop
  end if
  if ( a_FlagVMRSetFromFile(m) ) then
    if ( NVMRKeyMol1 /= 1 ) then
      write( 6, * ) 'For key molecule 1:'
      write( 6, * ) '  NVMRKeyMolX has to be 1, when FlagVMRSetFromFile is .true.'
      stop
    end if
  end if
  IndexKeyMol1 = m
  !
  do m = 1, NMol
    if ( m_MolNum(m) == KeyMol2Num ) exit
  end do
  if ( m > NMol ) then
    write( 6, * ) 'Unable to find KeyMol2=', KeyMol1Num, ' in MolNum.'
    write( 6, * ) '  MolNum = ', m_MolNum
    stop
  end if
  if ( a_FlagVMRSetFromFile(m) ) then
    if ( NVMRKeyMol2 /= 1 ) then
      write( 6, * ) 'For key molecule 2:'
      write( 6, * ) '  NVMRKeyMolX has to be 1, when FlagVMRSetFromFile is .true.'
      stop
    end if
  end if
  IndexKeyMol2 = m
  !
  do m = 1, NMol
    if ( m_MolNum(m) == KeyMol3Num ) exit
  end do
  if ( m > NMol ) then
    write( 6, * ) 'Unable to find KeyMol3=', KeyMol1Num, ' in MolNum.'
    write( 6, * ) '  MolNum = ', m_MolNum
    stop
  end if
  if ( a_FlagVMRSetFromFile(m) ) then
    if ( NVMRKeyMol3 /= 1 ) then
      write( 6, * ) 'For key molecule 3:'
      write( 6, * ) '  NVMRKeyMolX has to be 1, when FlagVMRSetFromFile is .true.'
      stop
    end if
  end if
  IndexKeyMol3 = m


  allocate( r_Press(NPress) )
  allocate( r_Temp (NPress) )

  do k = 1, NPress
!!$    Press = 10.0d0**( log10( PressMax ) - dble( k - 1 ) / dble(NPressPer10) )
    if ( FlagPressZero ) then
      if ( k < NPress ) then
        Press = &
          &   log( PressMin / PressMax ) / dble(NPress-1-1) * dble( k-1 ) &
          & + log( PressMax )
        Press = exp( Press )
      else
        Press = 0.0d0
      end if
    else
      Press = &
        &   log( PressMin / PressMax ) / dble(NPress-1  ) * dble( k-1 ) &
        & + log( PressMax )
      Press = exp( Press )
    end if

    r_Press(k) = Press
  end do

!!$  do m = 1+1, NMol
!!$    do k = 1, NPress
!!$      if ( m == 2 ) then
!!$        rm_VMR(k,m) = VMRCO2
!!$      end if
!!$      if ( m == 3 ) then
!!$        rm_VMR(k,m) = VMRO3
!!$      end if
!!$      if ( m > 3 ) then
!!$        rm_VMR(k,m) = 0.0d0
!!$      end if
!!$    end do
!!$  end do



  FlagReadVMRFile = .false.
  do m = 1, NMol
    if ( a_FlagVMRSetFromFile(m) ) then
      FlagReadVMRFile = .true.
    end if
  end do
  if ( FlagReadVMRFile ) then
    Mode = "read"
    call ni3_open( NcFNVMR, Mode, NcID )
    Name = "MolNum"
    call ni3_inq_dimlen( NcID, Name, NMolNumVMRFile )
    Name = "Press"
    call ni3_inq_dimlen( NcID, Name, NPressVMRFile )
    !
    allocate( m_MolNumVMRFile( NMolNumVMRFile ) )
    allocate( r_PressVMRFile ( NPressVMRFile ) )
    allocate( rm_VMRFile     ( NPressVMRFile, NMolNumVMRFile ) )
    !
    Name = "MolNum"
    call ni3_get_var( NcID, Name, m_MolNumVMRFile )
    Name = "Press"
    call ni3_get_var( NcID, Name, r_PressVMRFile )
    Name = "VMR"
    call ni3_get_var( NcID, Name, rm_VMRFile )
    !
    call ni3_close( NcID )

    do m = 1, NMol
      if ( a_FlagVMRSetFromFile(m) ) then

        do m2 = 1, NMolNumVMRFile
          if ( m_MolNum(m) == m_MolNumVMRFile(m2) ) then
            exit
          end if
        end do
        if ( m2 > NMolNumVMRFile ) then
          write( 6, * ) 'Unable to find molecular number ', m, ' in file.'
          stop
        end if
        call Interpolate1D( &
          & NPressVMRFile, r_PressVMRFile, rm_VMRFile(:,m2), &
          & NPress, r_Press,                                 &
          & rm_VMR(:,m)                                      &
          & )

      end if
    end do

    deallocate( m_MolNumVMRFile )
    deallocate( r_PressVMRFile  )
    deallocate( rm_VMRFile      )
  end if



  if ( FlagVMRZeroKeyMol1 ) then
    iVMRKeyMol1s = 0
    iVMRKeyMol1e = NVMRKeyMol1 - 1
  else
    iVMRKeyMol1s = 1
    iVMRKeyMol1e = NVMRKeyMol1
  end if
  if ( FlagVMRZeroKeyMol2 ) then
    iVMRKeyMol2s = 0
    iVMRKeyMol2e = NVMRKeyMol2 - 1
  else
    iVMRKeyMol2s = 1
    iVMRKeyMol2e = NVMRKeyMol2
  end if
  if ( FlagVMRZeroKeyMol3 ) then
    iVMRKeyMol3s = 0
    iVMRKeyMol3e = NVMRKeyMol3 - 1
  else
    iVMRKeyMol3s = 1
    iVMRKeyMol3e = NVMRKeyMol3
  end if


  do iVMRKeyMol3 = iVMRKeyMol3s, iVMRKeyMol3e
    do iVMRKeyMol2 = iVMRKeyMol2s, iVMRKeyMol2e
      do iVMRKeyMol1 = iVMRKeyMol1s, iVMRKeyMol1e

        if ( ( iVMRKeyMol3 == iVMRKeyMol3s ) .and. &
          &  ( iVMRKeyMol2 == iVMRKeyMol2s ) .and. &
          &  ( iVMRKeyMol1 == iVMRKeyMol1s ) ) then
          FlagWarning = .true.
        else
          FlagWarning = .false.
        end if

        ! mixing ratio of key mol. 1
        if ( a_FlagVMRSetFromFile(IndexKeyMol1) ) then
          ! VMR has already been set.
        else
          call SetVMRKeyMol( &
            & FlagWarning, KeyMol1Num, &
            & iVMRKeyMol1s, iVMRKeyMol1e, iVMRKeyMol1, &
            & FlagVMRZeroKeyMol1, VMR( IndexKeyMol1 ), &
            & VMRMaxKeyMol1, VMRMinKeyMol1, NPress, &
            & rm_VMR(:,IndexKeyMol1) &
            & )
        end if

        ! mixing ratio of key mol. 2
        if ( a_FlagVMRSetFromFile(IndexKeyMol2) ) then
          ! VMR has already been set.
        else
          call SetVMRKeyMol( &
            & FlagWarning, KeyMol2Num, &
            & iVMRKeyMol2s, iVMRKeyMol2e, iVMRKeyMol2, &
            & FlagVMRZeroKeyMol2, VMR( IndexKeyMol2 ), &
            & VMRMaxKeyMol2, VMRMinKeyMol2, NPress, &
            & rm_VMR(:,IndexKeyMol2) &
            & )
        end if


        ! mixing ratio of key mol. 3
        if ( a_FlagVMRSetFromFile(IndexKeyMol3) ) then
          ! VMR has already been set.
        else
          call SetVMRKeyMol( &
            & FlagWarning, KeyMol3Num, &
            & iVMRKeyMol3s, iVMRKeyMol3e, iVMRKeyMol3, &
            & FlagVMRZeroKeyMol3, VMR( IndexKeyMol3 ), &
            & VMRMaxKeyMol3, VMRMinKeyMol3, NPress, &
            & rm_VMR(:,IndexKeyMol3) &
            & )
        end if

        do iTemp = 1, NTemp

          Temp = ( TempMax - TempMin ) / ( NTemp - 1 ) * ( iTemp - 1 ) + TempMin

          r_Temp = Temp


          Ps = r_Press(1)
          Ts = Temp


          write( NcFN, '(a,i3.3,3(a,i2.2,a,i3.3),a)' ) &
            & 'data_profile/Profile_T-', iTemp, &
            & '_M', KeyMol1Num, '-', iVMRKeyMol1, &
            & '_M', KeyMol2Num, '-', iVMRKeyMol2, &
            & '_M', KeyMol3Num, '-', iVMRKeyMol3, &
            & '.nc'


          ! Output

          Mode = "new"
          call ni3_open( NcFN, Mode, NcID )

          Name     = "Press"
          StdName  = Name
          LongName = "Pressure"
          Units    = "Pa"

          call ni3_set_dim( NcID, Name, NF90_DOUBLE, r_Press, &
            & StdName, LongName, Units )
          Name     = "MolNum"
          StdName  = Name
          LongName = "Molecular number"
          Units    = "1"
          call ni3_set_dim( NcID, Name, NF90_DOUBLE, m_MolNum, &
            & StdName, LongName, Units )


          NDims = 1
          allocate( a_DimNames( NDims ) )
          !
          Name     = "Temp"
          StdName  = Name
          LongName = "Temperature"
          Units    = "K"
          a_DimNames( 1 ) = "Press"
          call ni3_def_var( NcID, Name, NF90_DOUBLE, NDims, a_DimNames, &
            & StdName, LongName, Units )
          call ni3_put_var( NcID, Name, r_Temp )
          !
          deallocate( a_DimNames )


          NDims = 2
          allocate( a_DimNames( NDims ) )
          !
          Name     = "VMR"
          StdName  = Name
          LongName = "Volume mixing ratio"
          Units    = "1"
          a_DimNames( 1 ) = "Press"
          a_DimNames( 2 ) = "MolNum"
          call ni3_def_var( NcID, Name, NF90_DOUBLE, NDims, a_DimNames, &
            & StdName, LongName, Units )
          call ni3_put_var( NcID, Name, rm_VMR )
          !
          deallocate( a_DimNames )

          !-------------------

          NDims = 0
          allocate( a_DimNames( NDims ) )
          !
          Name     = "Ts"
          StdName  = Name
          LongName = "Surface temperature"
          Units    = "K"
          call ni3_def_var( NcID, Name, NF90_DOUBLE, NDims, a_DimNames, &
            & StdName, LongName, Units )
          call ni3_put_var( NcID, Name, Ts )
          !
          Name     = "Ps"
          StdName  = Name
          LongName = "Surface pressure"
          Units    = "Pa"
          call ni3_def_var( NcID, Name, NF90_DOUBLE, NDims, a_DimNames, &
            & StdName, LongName, Units )
          call ni3_put_var( NcID, Name, Ps )
          !
          deallocate( a_DimNames )

          call ni3_close( NcID )

        end do
      end do
    end do
  end do


  stop

  !----------------------------------------------------------------------------

contains

  !----------------------------------------------------------------------------

  subroutine SetVMRKeyMol( &
    & FlagWarning, KeyMolNum, &
    & iVMRs, iVMRe, iVMR, &
    & FlagVMRZero, VMR, VMRMax, VMRMin, NPress, &
    & r_VMR &
    & )

    logical, intent(in ) :: FlagWarning
    integer, intent(in ) :: KeyMolNum
    integer, intent(in ) :: iVMRs
    integer, intent(in ) :: iVMRe
    integer, intent(in ) :: iVMR
    logical, intent(in ) :: FlagVMRZero
    real(8), intent(in ) :: VMR
    real(8), intent(in ) :: VMRMax
    real(8), intent(in ) :: VMRMin
    integer, intent(in ) :: NPress
    real(8), intent(out) :: r_VMR(NPress)

    ! Local variables
    !
    real(8) :: DelLogVMR


    if ( ( iVMRe - iVMRs ) == 0 ) then
      r_VMR = VMR

      if ( FlagWarning ) then
        write( 6, * ) "VMR of KeyMol ", KeyMolNum, " is set to ", VMR
      end if
    else
      if ( iVMR == 0 ) then
        r_VMR = 0.0d0
      else
        if ( FlagVMRZero ) then
          DelLogVMR = &
            &   log( VMRMax / VMRMin ) &
            & / dble( iVMRe - iVMRs - 1 )
        else
          DelLogVMR = &
            &   log( VMRMax / VMRMin ) &
            & / dble( iVMRe - iVMRs )
        end if
        r_VMR = &
          &   DelLogVMR * dble( iVMR - 1 ) &
          & + log( VMRMin )
        r_VMR = exp( r_VMR )
      end if
    end if

  end subroutine SetVMRKeyMol

  !----------------------------------------------------------------------------

  subroutine Interpolate1D( &
    & NLev, a_Press, a_Array,         &
    & kmax, z_Press,                  &
    & z_Array                         &
    & )

    integer , intent(in ) :: NLev
    real(DP), intent(in ) :: a_Press  (1:NLev)
    real(DP), intent(in ) :: a_Array  (1:NLev)
    integer , intent(in ) :: kmax
    real(DP), intent(in ) :: z_Press(1:kmax)
    real(DP), intent(out) :: z_Array(1:kmax)


    !
    ! local variables
    !
    integer :: k
    integer :: kk


    do k = 1, kmax

      if( z_Press(k) <= a_Press(NLev) ) then
        z_Array(k) = a_Array(NLev)
      else if( z_Press(k) >= a_Press(1) ) then
        z_Array(k) = a_Array(1)
      else
        search_loop : do kk = 2, NLev
          if( a_Press( kk ) < z_Press(k) ) exit search_loop
        end do search_loop
        if( kk > NLev ) &
          stop 'Unexpected error in setting temperature profile'
        z_Array(k) =                                     &
          &   ( a_Array( kk ) - a_Array( kk-1 ) )        &
          & / ( log( a_Press(kk  ) / a_Press( kk-1 ) ) ) &
          & * ( log( z_Press(k   ) / a_Press( kk-1 ) ) ) &
          & + a_Array( kk-1 )
      end if

    end do


  end subroutine Interpolate1D

  !----------------------------------------------------------------------------

end program main
