program main

  use vtype_module

  use fi_module

  use constants0, only : PI

  use rad_planetconst, only : &
    & RadPlanetConstInit, &
    & Grav,            &
    & Cp,              &
    & MolWtNonRadAct

  use gauss_quad, only : GauLeg

  use rad_flux, only : RadFluxCalcFlux, RadFluxInit, RadFluxFinalize, RadFluxOutputInit, RadFluxOutputFinalize
  use rad_kd_utils, only : RadKDUtilsCalcTendency
  use rad_output, only : RadOutputFluxTendency

  use hitranconst, only : m_HITRANMolWt

  use convection, only : &
    & ConvCalcRadTendency, &
    & ConvCalcRadConvTendency

  use io_profile, only : &
    & IOProfileInqDimLen, &
    & IOProfileRead, &
    & IOProfileReadPtcl, &
    & IOProfileOutput


  implicit none


  character(stdstr) :: CtlFN0 = 'planetconst.conf'
  real(DP)          :: GravSet
  real(DP)          :: CpSet
  real(DP)          :: MolWtNonRadActSet

  character(extstr) :: CtlFN3 = 'calc_rte.conf'
  character(extstr) :: CtlFN4 = 'calc_rce.conf'


  character(len=extstr) :: GasOptPropNcFN
  character(len=extstr) :: PtclOptPropNcFN
  character(len=extstr) :: KDOptPropNcFN

  character(len=extstr)     :: InitFileName
  character(len=extstr)     :: InFileName
  character(len=extstr)     :: PressName
  character(len=extstr)     :: MolNumName
  character(len=extstr)     :: TempName
  character(len=extstr)     :: SurfTempName
  character(len=extstr)     :: VMRName


  logical           :: FlagOutSpeFlux = .false.
  character(stdstr) :: OutSpeFluxNcFN
  character(stdstr) :: OutRadFluxNcFN
  character(stdstr) :: OutRadTendencyNcFN
  character(stdstr) :: OutTendenciesNcFN

  real(dp)       :: FilterCoef


  integer           :: iostat

  real(dp), allocatable :: r_Sigma(:)
  real(dp), allocatable :: z_DelAtmMass(:)
  real(dp), allocatable :: rm_MMR        (:,:)
  real(dp), allocatable :: r_MMRNonRadAct(:)
  real(dp), allocatable :: zm_MMR        (:,:)
  real(dp), allocatable :: z_MMRNonRadAct(:)
  real(dp), allocatable :: m_AtmMassRadAct(:)
  real(dp)              :: AtmMassDry
  real(dp)              :: AtmMassNonRadAct
  real(dp)              :: AtmMassTot
  real(dp)              :: AtmMassDrySave
  real(dp)              :: AtmMassNonRadActSave
  real(dp), allocatable :: m_AtmMassRadActSave(:)

  real(dp), allocatable :: r_DelTempRC    (:)
  real(dp)              :: DelSurfTemp
  real(dp), allocatable :: r_DelTempFilter(:)
  real(dp), allocatable :: r_DelVMRH2OVap (:)


  integer, parameter :: MaxNmlArySize = 64


  integer                       :: kmax

  !
  ! NMolNum : number of radiatively active species
  !
  integer                        :: NMolNum


  real(dp)                           :: DelWaveNum
  real(dp)                           :: WaveNumS
  real(dp)                           :: WaveNumE


  integer(i4b)              :: FileUnit

  logical                   :: FlagConvection
  logical                   :: FlagMoistConvection


  real(dp)    , allocatable :: r_Press (:)
  integer     , allocatable :: m_MolNum(:)
  real(dp)    , allocatable :: rm_VMR  (:,:)

  real(dp)    , allocatable :: r_Temp(:)
  real(dp)                  :: SurfTemp
  real(dp)    , allocatable :: z_Press(:)

  real(dp)    , allocatable :: r_TempBefConv(:)
  real(dp)    , allocatable :: r_TempB(:)
  real(dp)                  :: SurfTempB

  integer              , parameter   :: NChar = stdstr
  integer                            :: NPtcl
  integer              , allocatable :: a_PtclNum    (:)
  character(len=NChar) , allocatable :: a_PtclName   (:)
  real(8)              , allocatable :: a_PtclDens   (:)
  real(8)              , allocatable :: za_PtclEffRad(:,:)
  real(8)              , allocatable :: za_PtclMMR   (:,:)
  real(8)              , allocatable :: za_DelPtclNum(:,:)

  real(8), allocatable :: r_MeanMolWt(:)
  real(8), allocatable :: r_VMRNonRadAct(:)
  real(8), allocatable :: z_DTempDt      (:)
  real(8), allocatable :: z_DTempDtLW    (:)
  real(8), allocatable :: z_DTempDtSW    (:)
  real(8), allocatable :: r_RadUwFlux    (:)
  real(8), allocatable :: r_RadDwFlux    (:)
  real(DP), allocatable :: ra_DelRadUwFlux(:,:)
  real(DP), allocatable :: ra_DelRadDwFlux(:,:)

  real(DP), allocatable :: r_RadUwFluxLW    (:)
  real(DP), allocatable :: r_RadDwFluxLW    (:)
  real(DP), allocatable :: r_RadUwFluxSW    (:)
  real(DP), allocatable :: r_RadDwFluxSW    (:)
  real(DP), allocatable :: ra_DelRadUwFluxLW(:,:)
  real(DP), allocatable :: ra_DelRadDwFluxLW(:,:)
  real(DP), allocatable :: ra_DelRadUwFluxSW(:,:)
  real(DP), allocatable :: ra_DelRadDwFluxSW(:,:)

  real(DP), allocatable :: r_RadUwFluxEach      (:)
  real(DP), allocatable :: r_RadDwFluxEach      (:)
  real(DP), allocatable :: ra_DelRadUwFluxEach  (:,:)
  real(DP), allocatable :: ra_DelRadDwFluxEach  (:,:)
  real(DP), allocatable :: r_RadUwFluxLWEach    (:)
  real(DP), allocatable :: r_RadDwFluxLWEach    (:)
  real(DP), allocatable :: r_RadUwFluxSWEach    (:)
  real(DP), allocatable :: r_RadDwFluxSWEach    (:)
  real(DP), allocatable :: ra_DelRadUwFluxLWEach(:,:)
  real(DP), allocatable :: ra_DelRadDwFluxLWEach(:,:)
  real(DP), allocatable :: ra_DelRadUwFluxSWEach(:,:)
  real(DP), allocatable :: ra_DelRadDwFluxSWEach(:,:)

  real(8), allocatable :: r_RadUwFluxTentative  (:)
  real(8), allocatable :: r_RadDwFluxTentative  (:)
  real(8), allocatable :: r_RadUwFluxLWTentative(:)
  real(8), allocatable :: r_RadDwFluxLWTentative(:)
  real(8), allocatable :: r_RadUwFluxSWTentative(:)
  real(8), allocatable :: r_RadDwFluxSWTentative(:)

  real(DP)              :: StrFluxTOA
  character(len=extstr) :: StrSpeNcFN

  real(DP) :: AlbedoSW
  real(DP) :: AlbedoLW
  real(DP) :: WaveNumAlbedoSwitch

  integer               :: NGQ
  real(DP), allocatable :: a_GQMu    (:)
  real(DP), allocatable :: a_GQWeight(:)

  integer               :: NGQLoop
  real(DP)              :: SZADegEach

  real(DP) :: SZADeg

  real(DP) :: WNIntegStart
  real(DP) :: WNIntegEnd


  character(extstr)       :: OutFN
  character(extstr)       :: OutDir
  character(extstr)       :: OutNcFN = 'profile.nc'

!!$  logical  :: FlagUpdateSurfTemp = .true.
  logical  :: FlagAdjustAtmMass  = .true.
  real(DP) :: SensHeatFlux
!!$  real(DP) :: SurfNetUwFlux
  real(DP) :: SurfHeatCap

  integer  :: NTime
  real(DP) :: DelTime

  integer :: k
  integer :: m
  integer :: n
  integer :: iTime
  integer :: OutputStep


  namelist /planetconst_nml/ &
    & GravSet, CpSet, MolWtNonRadActSet
  namelist /wn_nml/ &
    & DelWaveNum, WaveNumS, WaveNumE

  namelist /rce_nml/ &
    & KDOptPropNcFN, &
    & FlagOutSpeFlux, &
    & DelTime, &
    & SurfHeatCap, &
    & NTime, &
    & OutputStep, &
    & FlagConvection, &
    & FlagMoistConvection, &
    & FilterCoef, &
    & InitFileName, &
    & PressName, MolNumName, TempName, VMRName, &
    & OutDir
!!$    & SurfTempName

  namelist /rte_nml/ &
    & StrFluxTOA, &
    & StrSpeNcFN, &
    & AlbedoSW, &
    & AlbedoLW, &
    & WaveNumAlbedoSwitch, &
    & NGQ, &
    & SZADeg, &
    & WNIntegStart, &
    & WNIntegEnd



  NTime       = 100
  DelTime     = 84600.0d0 * 10
  OutputStep  = 1
  SurfHeatCap = 4.217d3 * 1.0d3 * 10.0d0  ! [J K-1 kg-1] * [kg m-3] * [m]

  KDOptPropNcFN      = ''
  OutRadFluxNcFN     = 'out/flux.nc'
  OutRadTendencyNcFN = 'out/tendency.nc'
  FlagOutSpeFlux     = .false.
  OutSpeFluxNcFN     = 'out/flux_spectrum.nc'
  OutTendenciesNcFN  = 'out/tendencies.nc'

  FlagConvection      = .true.
  FlagMoistConvection = .true.
  FilterCoef          = 0.0d0

  !======================
  InitFileName = "./data.nc.nc"
  PressName    = "Press"
  MolNumName   = "MolNum"
  TempName     = "Temp"
  SurfTempName = "Ts"
  VMRName      = "VMR"
  !======================

  OutDir = "out"


!!$  StrFluxTOA          = 1370.0d0
  StrFluxTOA          =    0.0d0
  StrSpeNcFN          = "strspe.nc"
  AlbedoSW            = 0.2d0
  AlbedoLW            = 0.0d0
  WaveNumAlbedoSwitch = 3250.0d2
  NGQ                 =   -1
  SZADeg              =    0.0d0

  WNIntegStart        = -1.0d0
  WNIntegEnd          = -1.0d0


  ! Read a namelist file

  call fi_open( CtlFN0, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = planetconst_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist, planetconst_nml, is something wrong.'
  end if
  write( 6, nml = planetconst_nml )
  !
  close( FileUnit )

  call fi_open( CtlFN4, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = rce_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in rce_nml is something wrong.'
  end if
  write( 6, nml = rce_nml )
  !
  close( FileUnit )

  call fi_open( CtlFN3, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = rte_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in rte_nml is something wrong.'
  end if
  write( 6, nml = rte_nml )
  !
  close( FileUnit )


  ! Initialize planetary constants
  !
  call RadPlanetConstInit( &
    & GravSet           = GravSet,           &
    & CpSet             = CpSet,             &
    & MolWtNonRadActSet = MolWtNonRadActSet  &
    & )


  InFileName = InitFileName



  call IOProfileInqDimLen(      &
    & InFileName, PressName, &
    & kmax &
    & )
  kmax = kmax - 1
  call IOProfileInqDimLen(      &
    & InFileName, MolNumName, &
    & NMolNum &
    & )
  allocate( r_Press (0:kmax   ) )
  allocate( m_MolNum(1:NMolNum) )
  allocate( r_Temp  (0:kmax   ) )
  allocate( rm_VMR  (0:kmax   ,1:NMolNum) )
  call IOProfileRead(      &
    & InFileName, PressName, MolNumName, TempName, SurfTempName, VMRName, &
    & kmax+1, NMolNum, &
    & r_Press, m_MolNum, r_Temp, SurfTemp, rm_VMR     &
    & )

!!$  SurfTemp = r_Temp(0)


  allocate( r_MeanMolWt   (0:kmax) )
  allocate( r_VMRNonRadAct(0:kmax) )

  allocate( r_DelTempRC    (0:kmax) )
  allocate( r_DelTempFilter(0:kmax) )
  allocate( r_DelVMRH2OVap (0:kmax) )

  allocate( r_Sigma     (0:kmax) )
  allocate( z_DelAtmMass(1:kmax) )
  allocate( rm_MMR        (0:kmax,1:NMolNum) )
  allocate( r_MMRNonRadAct(0:kmax) )
  allocate( zm_MMR        (1:kmax,1:NMolNum) )
  allocate( z_MMRNonRadAct(1:kmax) )
  allocate( m_AtmMassRadAct    (1:NMolNum) )
  allocate( m_AtmMassRadActSave(1:NMolNum) )
  !
  r_Sigma = r_Press / r_Press(0)


  do k = 1, kmax
    z_DelAtmMass(k) = ( r_Press(k-1) - r_Press(k) ) / Grav
  end do
!!$  AtmMassTot = 0.0d0
!!$  do k = 1, kmax
!!$    AtmMassTot = AtmMassTot + z_DelAtmMass(k)
!!$  end do

  ! volume mixing ratio of non-radiative active specie
  r_VMRNonRadAct = 1.0d0
  do m = 1, NMolNum
    r_VMRNonRadAct = r_VMRNonRadAct - rm_VMR(:,m)
  end do
  ! mean molecular weight
  call CalcMeanMolWt( &
    & kmax, NMolNum, m_MolNum, rm_VMR, r_VMRNonRadAct, &
    & r_MeanMolWt, &
    & .true. &
    & )
  ! mass mixing ratio
  !   non-radiatively active specie
  r_MMRNonRadAct = r_VMRNonRadAct * MolWtNonRadAct / r_MeanMolWt
  do k = 1, kmax
    z_MMRNonRadAct(k) = ( r_MMRNonRadAct(k-1) + r_MMRNonRadAct(k) ) / 2.0d0
  end do
  !   radiatively active specie
  do m = 1, NMolNum
    rm_MMR(:,m) = rm_VMR(:,m) * m_HITRANMolWt(m_MolNum(m)) / r_MeanMolWt
  end do
  do m = 1, NMolNum
    do k = 1, kmax
      zm_MMR(k,m) = ( rm_MMR(k-1,m) + rm_MMR(k,m) ) / 2.0d0
    end do
  end do
  ! column mass
  !   non-radiatively active specie
  AtmMassNonRadAct = 0.0d0
  do k = 1, kmax
    AtmMassNonRadAct = AtmMassNonRadAct &
      & + z_DelAtmMass(k) * z_MMRNonRadAct(k)
  end do
  !   radiatively active specie
  m_AtmMassRadAct = 0.0d0
  do m = 1, NMolNum
    do k = 1, kmax
      m_AtmMassRadAct(m) = m_AtmMassRadAct(m) &
        & + z_DelAtmMass(k) * zm_MMR(k,m)
    end do
  end do
  !   total
  AtmMassTot = AtmMassNonRadAct
  do m = 1, NMolNum
    AtmMassTot = AtmMassTot + m_AtmMassRadAct(m)
  end do
  !   dry
  AtmMassDry = AtmMassTot - m_AtmMassRadAct(1)
  ! save
  AtmMassNonRadActSave = AtmMassNonRadAct
  m_AtmMassRadActSave  = m_AtmMassRadAct
  AtmMassDrySave       = AtmMassDry


  ! temporarily set
!!$  NPtcl              = 2
!!$  do m = 1, NPtcl
!!$    a_PtclNum(m) = m
!!$  end do
  !
  call IOProfileInqDimLen(      &
    & InFileName, "PtclNum", &
    & NPtcl &
    & )
  !
  allocate( z_Press (1:kmax) )
  !
  allocate( a_PtclNum (NPtcl) )
  allocate( a_PtclName(NPtcl) )
  allocate( za_PtclEffRad(kmax,NPtcl) )
  allocate( a_PtclDens   (NPtcl) )
  allocate( za_PtclMMR   (kmax,NPtcl) )
  allocate( za_DelPtclNum(kmax,NPtcl) )
  !
!!$  do k = 1, kmax
!!$    if ( r_Press(k-1) == 0.0d0 ) then
!!$      z_Press(k) = ( r_Press(k-1) + r_Press(k) ) / 2.0d0
!!$    else
!!$      z_Press(k) = sqrt( r_Press(k-1) * r_Press(k) )
!!$    end if
!!$  end do
  !
!!$  a_PtclName(1)      = "water"
!!$  a_PtclName(2)      = "ice"
!!$  za_PtclEffRad(:,1) = 10.0d-6
!!$  za_PtclEffRad(:,2) = 50.0d-6
!!$  za_DelPtclNum      = 0.0d0
  !
  call IOProfileReadPtcl(      &
    & InFileName, &
    & kmax, NPtcl, &
    & z_Press, a_PtclNum, a_PtclName, a_PtclDens, za_PtclEffRad, za_PtclMMR  &
    & )
  do m = 1, NPtcl
    do k = 1, kmax
      za_DelPtclNum(k,m) = za_PtclMMR(k,m) &
        & * ( r_Press(k-1) - r_Press(k) ) / Grav &
        & / ( 4.0d0 / 3.0d0 * PI * za_PtclEffRad(k,m)**3 * a_PtclDens(m) )
    end do
  end do


  allocate( r_TempBefConv(0:kmax) )
  allocate( r_TempB      (0:kmax) )

  allocate( z_DTempDt  (1:kmax) )
  allocate( z_DTempDtLW(1:kmax) )
  allocate( z_DTempDtSW(1:kmax) )
  allocate( r_RadUwFlux(0:kmax) )
  allocate( r_RadDwFlux(0:kmax) )
  allocate( ra_DelRadUwFlux(0:kmax,-1:1) )
  allocate( ra_DelRadDwFlux(0:kmax,-1:1) )

  allocate( r_RadUwFluxLW    (0:kmax) )
  allocate( r_RadDwFluxLW    (0:kmax) )
  allocate( r_RadUwFluxSW    (0:kmax) )
  allocate( r_RadDwFluxSW    (0:kmax) )
  allocate( ra_DelRadUwFluxLW(0:kmax,-1:1) )
  allocate( ra_DelRadDwFluxLW(0:kmax,-1:1) )
  allocate( ra_DelRadUwFluxSW(0:kmax,-1:1) )
  allocate( ra_DelRadDwFluxSW(0:kmax,-1:1) )

  allocate( r_RadUwFluxEach      (0:kmax) )
  allocate( r_RadDwFluxEach      (0:kmax) )
  allocate( ra_DelRadUwFluxEach  (0:kmax,-1:1) )
  allocate( ra_DelRadDwFluxEach  (0:kmax,-1:1) )
  allocate( r_RadUwFluxLWEach    (0:kmax) )
  allocate( r_RadDwFluxLWEach    (0:kmax) )
  allocate( r_RadUwFluxSWEach    (0:kmax) )
  allocate( r_RadDwFluxSWEach    (0:kmax) )
  allocate( ra_DelRadUwFluxLWEach(0:kmax,-1:1) )
  allocate( ra_DelRadDwFluxLWEach(0:kmax,-1:1) )
  allocate( ra_DelRadUwFluxSWEach(0:kmax,-1:1) )
  allocate( ra_DelRadDwFluxSWEach(0:kmax,-1:1) )

  allocate( r_RadUwFluxTentative  (0:kmax) )
  allocate( r_RadDwFluxTentative  (0:kmax) )
  allocate( r_RadUwFluxLWTentative(0:kmax) )
  allocate( r_RadDwFluxLWTentative(0:kmax) )
  allocate( r_RadUwFluxSWTentative(0:kmax) )
  allocate( r_RadDwFluxSWTentative(0:kmax) )

  if ( NGQ > 0 ) then
    NGQLoop = NGQ + 1
    allocate( a_GQMu    ( NGQ+1 ) )
    allocate( a_GQWeight( NGQ+1 ) )
    call GauLeg( 0.0d0, 1.0d0, NGQ, a_GQMu(1:NGQ), a_GQWeight(1:NGQ) )
    ! divid weights by 2 in order to consider night side
    a_GQWeight = a_GQWeight / 2.0d0
    ! night side
    a_GQMu    (NGQ+1) = -1.0d0
    a_GQWeight(NGQ+1) =  0.5d0
  else
    NGQLoop = 1
    allocate( a_GQMu    ( 1 ) )
    allocate( a_GQWeight( 1 ) )
    n = 1
    a_GQMu    (n) = cos( SZADeg * PI / 180.0d0 )
    a_GQWeight(n) = 1.0d0
  end if


  iTime = 0
  write( OutFN, '(a,i6.6,a)' ) &
    & trim( OutDir ) // "/" // OutNcFN(1:len_trim(OutNcFN)-3), iTime, '.nc'
!!$  call Output( &
!!$    & OutFN, &
!!$    & kmax, r_Press, r_Temp, NMolNum, m_MolNum, rm_VMR &
!!$    & )
  call IOProfileOutput( &
    & OutFN, &
    & kmax+1, NMolNum, r_Press, m_MolNum, &
    & r_Temp, rm_VMR, r_Press(0), SurfTemp, &
    & z_Press, &
    & NChar, NPtcl, a_PtclNum, a_PtclName, &
    & a_PtclDens, za_PtclEffRad, za_PtclMMR &
    & )

  PtclOptPropNcFN = ''


  do iTime = 1, NTime
    write( 6, * ) 'Loop: ', iTime

    r_TempB   = r_Temp
    SurfTempB = SurfTemp

    call CalcMeanMolWt( &
      & kmax, NMolNum, m_MolNum, rm_VMR, r_VMRNonRadAct, &
      & r_MeanMolWt, &
      & .true. &
      & )

    do k = 1, kmax
      z_DelAtmMass(k) = ( r_Press(k-1) - r_Press(k) ) / Grav
    end do
    AtmMassTot = 0.0d0
    do k = 1, kmax
      AtmMassTot = AtmMassTot + z_DelAtmMass(k)
    end do
    !
    r_MMRNonRadAct = r_VMRNonRadAct * MolWtNonRadAct / r_MeanMolWt
    do m = 1, NMolNum
      rm_MMR(:,m) = rm_VMR(:,m) * m_HITRANMolWt(m_MolNum(m)) / r_MeanMolWt
    end do
    do k = 1, kmax
      z_MMRNonRadAct(k) = ( r_MMRNonRadAct(k-1) + r_MMRNonRadAct(k) ) / 2.0d0
    end do
    do m = 1, NMolNum
      do k = 1, kmax
        zm_MMR(k,m) = ( rm_MMR(k-1,m) + rm_MMR(k,m) ) / 2.0d0
      end do
    end do
    AtmMassNonRadAct = 0.0d0
    do k = 1, kmax
      AtmMassNonRadAct = AtmMassNonRadAct &
        & + z_DelAtmMass(k) * z_MMRNonRadAct(k)
    end do
    m_AtmMassRadAct = 0.0d0
    do m = 1, NMolNum
      do k = 1, kmax
        m_AtmMassRadAct(m) = m_AtmMassRadAct(m) &
          & + z_DelAtmMass(k) * zm_MMR(k,m)
      end do
    end do


    GasOptPropNcFN  = KDOptPropNcFN


    if ( iTime == 1 ) then
      call RadFluxInit( &
        & Grav, &
        & WaveNumAlbedoSwitch, &
        & GasOptPropNcFN, PtclOptPropNcFN, &
        & StrSpeNcFN, &
        & Method = "KD" &
        & )
    end if

    if ( FlagOutSpeFlux ) then
      call RadFluxOutputInit( &
        & GasOptPropNcFN, &
        & OutSpeFluxNcFN, &
        & kmax, r_Press &
        & )
    end if
    r_RadUwFlux       = 0.0d0
    r_RadDwFlux       = 0.0d0
    ra_DelRadUwFlux   = 0.0d0
    ra_DelRadDwFlux   = 0.0d0
    r_RadUwFluxLW     = 0.0d0
    r_RadDwFluxLW     = 0.0d0
    r_RadUwFluxSW     = 0.0d0
    r_RadDwFluxSW     = 0.0d0
    ra_DelRadUwFluxLW = 0.0d0
    ra_DelRadDwFluxLW = 0.0d0
    ra_DelRadUwFluxSW = 0.0d0
    ra_DelRadDwFluxSW = 0.0d0
    if ( NGQ > 0 ) then
      write( 6, '(i3,a)' ) NGQ, '-point Gaussian quadrature for dayside + nightside:'
    end if
    do n = 1, NGQLoop
      SZADegEach = acos( a_GQMu(n) ) * 180.0d0/PI
      if ( NGQ > 0 ) then
        write( 6, * ) '   ', n, SZADegEach
      end if
      call RadFluxCalcFlux( &
        & kmax, NMolNum, &
        & m_MolNum, &
        & r_Press, &
        & SurfTemp, r_Temp, rm_VMR, &
        & NPtcl, a_PtclName, za_PtclEffRad, za_DelPtclNum, &
        & AlbedoSW, AlbedoLW, SZADegEach, &
        & StrFluxTOA, &
        & r_MeanMolWt, &
        & r_RadUwFluxEach, r_RadDwFluxEach, &
        & ra_DelRadUwFluxEach, ra_DelRadDwFluxEach, &
        & r_RadUwFluxLWEach, r_RadDwFluxLWEach, &
        & r_RadUwFluxSWEach, r_RadDwFluxSWEach, &
        & ra_DelRadUwFluxLWEach, ra_DelRadDwFluxLWEach, &
        & ra_DelRadUwFluxSWEach, ra_DelRadDwFluxSWEach, &
        & WNIntegStart, WNIntegEnd &
        & )
      r_RadUwFlux       = r_RadUwFlux       + r_RadUwFluxEach       * a_GQWeight(n)
      r_RadDwFlux       = r_RadDwFlux       + r_RadDwFluxEach       * a_GQWeight(n)
      ra_DelRadUwFlux   = ra_DelRadUwFlux   + ra_DelRadUwFluxEach   * a_GQWeight(n)
      ra_DelRadDwFlux   = ra_DelRadDwFlux   + ra_DelRadDwFluxEach   * a_GQWeight(n)
      r_RadUwFluxLW     = r_RadUwFluxLW     + r_RadUwFluxLWEach     * a_GQWeight(n)
      r_RadDwFluxLW     = r_RadDwFluxLW     + r_RadDwFluxLWEach     * a_GQWeight(n)
      r_RadUwFluxSW     = r_RadUwFluxSW     + r_RadUwFluxSWEach     * a_GQWeight(n)
      r_RadDwFluxSW     = r_RadDwFluxSW     + r_RadDwFluxSWEach     * a_GQWeight(n)
      ra_DelRadUwFluxLW = ra_DelRadUwFluxLW + ra_DelRadUwFluxLWEach * a_GQWeight(n)
      ra_DelRadDwFluxLW = ra_DelRadDwFluxLW + ra_DelRadDwFluxLWEach * a_GQWeight(n)
      ra_DelRadUwFluxSW = ra_DelRadUwFluxSW + ra_DelRadUwFluxSWEach * a_GQWeight(n)
      ra_DelRadDwFluxSW = ra_DelRadDwFluxSW + ra_DelRadDwFluxSWEach * a_GQWeight(n)
    end do
    if ( FlagOutSpeFlux ) then
      call RadFluxOutputFinalize
    end if



    ! Old convection
    !
!!$    call RadKDUtilsCalcTendency( &
!!$      & Grav, Cp, &
!!$      & kmax, r_Press, &
!!$      & r_RadUwFlux, r_RadDwFlux, &
!!$      & z_DTempDt &
!!$      & )
!!$    call RadOutputFluxTendency( &
!!$      & OutFluxNcFN, OutTendencyNcFN,   &
!!$      & kmax, r_Press, z_Press, &
!!$      & r_RadUwFlux, r_RadDwFlux, z_DTempDt &
!!$      & )
!!$    !
!!$    do k = 0+1, kmax
!!$      r_Temp(k) = r_Temp(k) + z_DTempDt(k) * DelTime
!!$    end do
!!$
!!$    r_TempBefConv = r_Temp
!!$
!!$    call ConvectionTrial( &
!!$      & kmax, NMolNum, r_Press, r_MeanMolWt, &
!!$      & r_Temp, rm_VMR &
!!$      & )
!!$
!!$    SensHeatFlux = 0.0d0
!!$    do k = 0+1, kmax
!!$      if ( r_Temp(k) == r_TempBefConv(k) ) exit
!!$      SensHeatFlux = SensHeatFlux &
!!$        & + Cp * ( r_Temp(k) - r_TempBefConv(k) ) &
!!$        &   * ( r_Press(k-1) - r_Press(k) ) / Grav
!!$      !
!!$!      if ( z_DTempDt(k) > 0.0d0 ) exit
!!$!      SensHeatFlux = SensHeatFlux &
!!$!        & - Cp * z_DTempDt(k) &
!!$!        &   * ( r_Press(k-1) - r_Press(k) ) / Grav &
!!$!        &   * DelTime
!!$    end do
!!$    SensHeatFlux = SensHeatFlux / DelTime
!!$
!!$    write( 6, * ) "FlagUpdateSurfTemp: ", FlagUpdateSurfTemp
!!$    write( 6, * ) "Convection        : ", iTime, k, r_RadUwFlux(0)-r_RadDwFlux(0)+SensHeatFlux
!!$    write( 6, * ) "Flux              : ", r_RadUwFlux(0), r_RadDwFlux(0), SensHeatFlux
!!$
!!$    ! Update surface temperature
!!$    if ( FlagUpdateSurfTemp ) then
!!$      SurfNetUwFlux = r_RadUwFlux(0) - r_RadDwFlux(0) + SensHeatFlux
!!$      !
!!$      DelSurfTemp = - SurfNetUwFlux / ra_DelRadUwFlux(0,0)
!!$      DelSurfTemp = DelSurfTemp * 1.0d0 / 5.0d0
!!$      !
!!$      DelSurfTemp = - SurfNetUwFlux * DelTime / SurfHeatCap
!!$      !
!!$      SurfTemp = SurfTemp + DelSurfTemp
!!$      r_Temp(0) = SurfTemp
!!$    end if



    ! New convection and add radiation

    if ( FlagConvection ) then
      call ConvCalcRadConvTendency( &
        & FlagMoistConvection, &
        & Grav, Cp, SurfHeatCap, DelTime, &
        & kmax, r_Press, r_Temp, SurfTemp, rm_VMR(:,1), r_MeanMolWt, &
        & r_RadUwFlux, r_RadDwFlux, ra_DelRadUwFlux, ra_DelRadDwFlux, &
        & r_DelTempRC, DelSurfTemp, r_DelVMRH2OVap, SensHeatFlux &
        & )
    else
      call ConvCalcRadTendency( &
        & Grav, Cp, SurfHeatCap, DelTime, &
        & kmax, r_Press, r_Temp, &
        & r_RadUwFlux, r_RadDwFlux, ra_DelRadUwFlux, ra_DelRadDwFlux, &
        & r_DelTempRC, DelSurfTemp &
        & )
      r_DelVMRH2OVap = 0.0d0
      SensHeatFlux   = 0.0d0
    end if
    r_Temp      = r_Temp      + r_DelTempRC
    SurfTemp    = SurfTemp    + DelSurfTemp
    rm_VMR(:,1) = rm_VMR(:,1) + r_DelVMRH2OVap

    r_RadUwFluxTentative = &
      &   r_RadUwFlux &
      & + ra_DelRadUwFlux(:,-1)*(SurfTemp -SurfTempB ) &
      & + ra_DelRadUwFlux(:, 0)*(r_Temp(0)-r_TempB(0)) &
      & + ra_DelRadUwFlux(:, 1)*(r_Temp(1)-r_TempB(1))
    r_RadDwFluxTentative = &
      &   r_RadDwFlux &
      & + ra_DelRadDwFlux(:,-1)*(SurfTemp -SurfTempB ) &
      & + ra_DelRadDwFlux(:, 0)*(r_Temp(0)-r_TempB(0)) &
      & + ra_DelRadDwFlux(:, 1)*(r_Temp(1)-r_TempB(1))

    write( 6, * ) "Flux       : ", iTime, &
      &   r_RadUwFluxTentative(0)  &
      & - r_RadDwFluxTentative(0)  &
      & + SensHeatFlux,            &
      &   r_RadUwFluxTentative(0), &
      &   r_RadDwFluxTentative(0), &
      &   SensHeatFlux


    r_RadUwFluxLWTentative = &
      &   r_RadUwFluxLW &
      & + ra_DelRadUwFluxLW(:,-1)*(SurfTemp -SurfTempB ) &
      & + ra_DelRadUwFluxLW(:, 0)*(r_Temp(0)-r_TempB(0)) &
      & + ra_DelRadUwFluxLW(:, 1)*(r_Temp(1)-r_TempB(1))
    r_RadDwFluxLWTentative = &
      &   r_RadDwFluxLW &
      & + ra_DelRadDwFluxLW(:,-1)*(SurfTemp -SurfTempB ) &
      & + ra_DelRadDwFluxLW(:, 0)*(r_Temp(0)-r_TempB(0)) &
      & + ra_DelRadDwFluxLW(:, 1)*(r_Temp(1)-r_TempB(1))
    r_RadUwFluxSWTentative = &
      &   r_RadUwFluxSW &
      & + ra_DelRadUwFluxSW(:,-1)*(SurfTemp -SurfTempB ) &
      & + ra_DelRadUwFluxSW(:, 0)*(r_Temp(0)-r_TempB(0)) &
      & + ra_DelRadUwFluxSW(:, 1)*(r_Temp(1)-r_TempB(1))
    r_RadDwFluxSWTentative = &
      &   r_RadDwFluxSW &
      & + ra_DelRadDwFluxSW(:,-1)*(SurfTemp -SurfTempB ) &
      & + ra_DelRadDwFluxSW(:, 0)*(r_Temp(0)-r_TempB(0)) &
      & + ra_DelRadDwFluxSW(:, 1)*(r_Temp(1)-r_TempB(1))

    call RadKDUtilsCalcTendency( &
      & Grav, Cp, &
      & kmax, r_Press, &
      & r_RadUwFluxTentative, r_RadDwFluxTentative, &
      & z_DTempDt &
      & )
    call RadKDUtilsCalcTendency( &
      & Grav, Cp, &
      & kmax, r_Press, &
      & r_RadUwFluxLWTentative, r_RadDwFluxLWTentative, &
      & z_DTempDtLW &
      & )
    call RadKDUtilsCalcTendency( &
      & Grav, Cp, &
      & kmax, r_Press, &
      & r_RadUwFluxSWTentative, r_RadDwFluxSWTentative, &
      & z_DTempDtSW &
      & )
    call RadOutputFluxTendency( &
      & OutRadFluxNcFN, OutRadTendencyNcFN,   &
      & kmax, r_Press, z_Press, &
      & r_RadUwFluxTentative  , r_RadDwFluxTentative  , z_DTempDt  , &
      & r_RadUwFluxLWTentative, r_RadDwFluxLWTentative, z_DTempDtLW, &
      & r_RadUwFluxSWTentative, r_RadDwFluxSWTentative, z_DTempDtSW  &
      & )


    ! CO2 condensation temperature (Pollack et al., 1981)
!!$    r_Temp = max( r_Temp, 149.2d0 + 6.48d0 * log( 0.135d0 * r_Press * 1.0d-2 ) )
!!$    SurfTemp = max( SurfTemp, 149.2d0 + 6.48d0 * log( 0.135d0 * r_Press(0) * 1.0d-2 ) )


    if ( FlagAdjustAtmMass ) then


      ! Total mass adjustment
      !
      !   Estimate H2O mass change
      !
!!$      m = 1
!!$      r_MMRH2O = rm_VMR(:,m) * m_HITRANMolWt(m_MolNum(m)) / r_MeanMolWt
!!$      end do
!!$      do k = 1, kmax
!!$        z_MMRH2O(k) = ( r_MMRH2O(k-1) + r_MMRH2O(k) ) / 2.0d0
!!$      end do
!!$      DelAtmMassH2O = 0.0d0
!!$      do k = 1, kmax
!!$        DelAtmMassH2O = DelAtmMassH2O &
!!$          & + z_DelAtmMass(k) * ( z_MMRH2O(k) - zm_MMRB(k,m) )
!!$      end do
!!$      !
!!$      AtmMassTot = AtmMassTot + DelAtmMassH2O
      !
!!$      call CalcMeanMolWt( &
!!$        & kmax, NMolNum, m_MolNum, rm_VMR, r_VMRNonRadAct, &
!!$        & r_MeanMolWt, &
!!$        & .false. &
!!$        & )
!!$      !
!!$      r_MMRNonRadAct = r_VMRNonRadAct * MolWtNonRadAct / r_MeanMolWt
!!$      do m = 1, NMolNum
!!$        rm_MMR(:,m) = rm_VMR(:,m) * m_HITRANMolWt(m_MolNum(m)) / r_MeanMolWt
!!$      end do
!!$      do k = 1, kmax
!!$        z_MMRNonRadAct(k) = ( r_MMRNonRadAct(k-1) + r_MMRNonRadAct(k) ) / 2.0d0
!!$      end do
!!$      do m = 1, NMolNum
!!$        do k = 1, kmax
!!$          zm_MMR(k,m) = ( rm_MMR(k-1,m) + rm_MMR(k,m) ) / 2.0d0
!!$        end do
!!$      end do
!!$      AtmMassNonRadAct = 0.0d0
!!$      do k = 1, kmax
!!$        AtmMassNonRadAct = AtmMassNonRadAct &
!!$          & + z_DelAtmMass(k) * z_MMRNonRadAct(k)
!!$      end do
!!$      m_AtmMassRadAct = 0.0d0
!!$      do m = 1, NMolNum
!!$        do k = 1, kmax
!!$          m_AtmMassRadAct(m) = m_AtmMassRadAct(m) &
!!$            & + z_DelAtmMass(k) * zm_MMR(k,m)
!!$        end do
!!$      end do
!!$      !
!!$      AtmMassTot = AtmMassDrySave + m_AtmMassRadAct(1)


      m = 1
      ! H2O
      !   mass mixing ratio
      rm_MMR(:,m) = rm_VMR(:,m) * m_HITRANMolWt(m_MolNum(m)) / r_MeanMolWt
      do k = 1, kmax
        zm_MMR(k,m) = ( rm_MMR(k-1,m) + rm_MMR(k,m) ) / 2.0d0
      end do
      !   mass
      m_AtmMassRadAct(m) = 0.0d0
      do k = 1, kmax
        m_AtmMassRadAct(m) = m_AtmMassRadAct(m) &
          & + z_DelAtmMass(k) * zm_MMR(k,m)
      end do
      !
      ! Total mass
      AtmMassTot = AtmMassDrySave + m_AtmMassRadAct(1)


      ! column mass of each component
      !   mean molecular weight
      call CalcMeanMolWt( &
        & kmax, NMolNum, m_MolNum, rm_VMR, r_VMRNonRadAct, &
        & r_MeanMolWt, &
        & .false. &
        & )
      ! mass mixing ratio
      !   non-radiatively active specie
      r_MMRNonRadAct = r_VMRNonRadAct * MolWtNonRadAct / r_MeanMolWt
      do k = 1, kmax
        z_MMRNonRadAct(k) = ( r_MMRNonRadAct(k-1) + r_MMRNonRadAct(k) ) / 2.0d0
      end do
      !   radiatively active specie
      do m = 1, NMolNum
        rm_MMR(:,m) = rm_VMR(:,m) * m_HITRANMolWt(m_MolNum(m)) / r_MeanMolWt
      end do
      do m = 1, NMolNum
        do k = 1, kmax
          zm_MMR(k,m) = ( rm_MMR(k-1,m) + rm_MMR(k,m) ) / 2.0d0
        end do
      end do
      ! column mass
      !   non-radiatively active specie
      AtmMassNonRadAct = 0.0d0
      do k = 1, kmax
        AtmMassNonRadAct = AtmMassNonRadAct &
          & + z_DelAtmMass(k) * z_MMRNonRadAct(k)
      end do
      !   radiatively active specie
      m_AtmMassRadAct = 0.0d0
      do m = 1, NMolNum
        do k = 1, kmax
          m_AtmMassRadAct(m) = m_AtmMassRadAct(m) &
            & + z_DelAtmMass(k) * zm_MMR(k,m)
        end do
      end do

      ! Pressure adjustment
      !
      r_Press(0) = AtmMassTot * Grav
      r_Press(1:kmax) = r_Press(0) * r_Sigma(1:kmax)
      do k = 1, kmax
        z_DelAtmMass(k) = ( r_Press(k-1) - r_Press(k) ) / Grav
      end do
      !
      do k = 1, kmax
        if ( r_Press(k-1) == 0.0d0 ) then
          z_Press(k) = ( r_Press(k-1) + r_Press(k) ) / 2.0d0
        else
          z_Press(k) = sqrt( r_Press(k-1) * r_Press(k) )
        end if
      end do

      ! Component mass adjustment
      !
      r_VMRNonRadAct = r_VMRNonRadAct * AtmMassNonRadActSave / AtmMassNonRadAct
      do m = 1+1, NMolNum
        ! H2O calculation is avoided
        if ( m_AtmMassRadAct(m) > 0.0d0 ) then
          rm_VMR(:,m) = rm_VMR(:,m) * m_AtmMassRadActSave(m) / m_AtmMassRadAct(m)
        end if
      end do

      write( 6, * ) 'Mass : ', AtmMassTot, AtmMassDrySave, AtmMassNonRadAct, m_AtmMassRadAct
    end if



    call Filter( &
      & FilterCoef, &
      & kmax, r_Press, &
      & r_Temp, &
      & r_DelTempFilter &
      & )

    if ( mod( iTime, OutputStep ) == 0 ) then
      write( OutFN, '(a,i6.6,a)' ) &
        & trim( OutDir ) // "/" // OutNcFN(1:len_trim(OutNcFN)-3), iTime, '.nc'
      call IOProfileOutput( &
        & OutFN, &
        & kmax+1, NMolNum, r_Press, m_MolNum, &
        & r_Temp, rm_VMR, r_Press(0), SurfTemp, &
        & z_Press, &
        & NChar, NPtcl, a_PtclNum, a_PtclName, &
        & a_PtclDens, za_PtclEffRad, za_PtclMMR &
        & )
    end if

    call OutputTendencies( &
      & OutTendenciesNcFN, &
      & DelTime, &
      & kmax, r_Press, &
      & r_DelTempRC, &
      & z_DTempDtLW, z_DTempDtSW, &
      & r_DelTempFilter,  &
      & DelSurfTemp  &
      & )


    AtmMassNonRadActSave = AtmMassNonRadAct
    m_AtmMassRadActSave  = m_AtmMassRadAct
    AtmMassDrySave       = AtmMassDry

  end do

  deallocate( a_GQMu     )
  deallocate( a_GQWeight )

  call RadFluxFinalize


  OutFN = trim( OutDir ) // "/" // OutNcFN
!!$  call Output( &
!!$    & OutFN, &
!!$    & kmax, r_Press, r_Temp, NMolNum, m_MolNum, rm_VMR &
!!$    & )
  call IOProfileOutput( &
    & OutFN, &
    & kmax+1, NMolNum, r_Press, m_MolNum, &
    & r_Temp, rm_VMR, r_Press(0), SurfTemp, &
    & z_Press, &
    & NChar, NPtcl, a_PtclNum, a_PtclName, &
    & a_PtclDens, za_PtclEffRad, za_PtclMMR &
    & )


  deallocate( r_Press  )
  deallocate( m_MolNum )
  deallocate( r_Temp   )
  deallocate( rm_VMR   )

  deallocate( z_Press  )

  deallocate( r_Sigma )

  deallocate( a_PtclName    )
  deallocate( za_PtclEffRad )
  deallocate( za_DelPtclNum )

  deallocate( r_TempBefConv   )

  deallocate( r_DelTempRC     )
  deallocate( r_DelTempFilter )
  deallocate( r_DelVMRH2OVap  )

  deallocate( r_MeanMolWt     )
  deallocate( z_DTempDt       )
  deallocate( r_RadUwFlux     )
  deallocate( r_RadDwFlux     )
  deallocate( ra_DelRadUwFlux )
  deallocate( ra_DelRadDwFlux )

  deallocate( r_RadUwFluxLW     )
  deallocate( r_RadDwFluxLW     )
  deallocate( r_RadUwFluxSW     )
  deallocate( r_RadDwFluxSW     )
  deallocate( ra_DelRadUwFluxLW )
  deallocate( ra_DelRadDwFluxLW )
  deallocate( ra_DelRadUwFluxSW )
  deallocate( ra_DelRadDwFluxSW )

  deallocate( r_RadUwFluxEach       )
  deallocate( r_RadDwFluxEach       )
  deallocate( ra_DelRadUwFluxEach   )
  deallocate( ra_DelRadDwFluxEach   )
  deallocate( r_RadUwFluxLWEach     )
  deallocate( r_RadDwFluxLWEach     )
  deallocate( r_RadUwFluxSWEach     )
  deallocate( r_RadDwFluxSWEach     )
  deallocate( ra_DelRadUwFluxLWEach )
  deallocate( ra_DelRadDwFluxLWEach )
  deallocate( ra_DelRadUwFluxSWEach )
  deallocate( ra_DelRadDwFluxSWEach )


  write( 6, * ) 'Finish'


end program main

!------------------------------------------------------------------------------

subroutine Filter( &
  & FilterCoef, &
  & kmax, r_Press, &
  & r_Temp, &
  & r_DelTemp &
  & )

  use vtype_module

  use rad_planetconst, only : &
    & Grav,            &
    & Cp

  implicit none

  real(DP), intent(in   ) :: FilterCoef
  integer , intent(in   ) :: kmax
  real(DP), intent(in   ) :: r_Press(0:kmax)
  real(DP), intent(inout) :: r_Temp     (0:kmax)
  real(DP), intent(  out) :: r_DelTemp  (0:kmax)


  ! Local variables
  !
  real(DP) :: r_TempB     (0:kmax)
  real(DP) :: z_DelAtmMass(1:kmax)
  real(DP) :: EneSumB
  real(DP) :: EneSum

  integer  :: k


  r_TempB = r_Temp

  do k = 1, kmax
    z_DelAtmMass(k) = ( r_Press(k-1) - r_Press(k) ) / Grav
  end do

  do k = 0+1, kmax-1
    if ( k == 1 ) then
      EneSumB =   0.0d0                              &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k  ) &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k+1)
    else
      EneSumB =   Cp * z_DelAtmMass(k) * r_Temp(k-1) &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k  ) &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k+1)
    end if
    r_Temp(k) = r_Temp(k) &
      & + FilterCoef * ( r_Temp(k-1) - 2.0d0 * r_Temp(k) + r_Temp(k+1) )
    if ( k == 1 ) then
      EneSum  =   0.0d0                              &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k  ) &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k+1)
      !
      r_Temp(k  ) = r_Temp(k  ) * EneSumB / EneSum
      r_Temp(k+1) = r_Temp(k+1) * EneSumB / EneSum
    else
      EneSum  =   Cp * z_DelAtmMass(k) * r_Temp(k-1) &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k  ) &
        &       + Cp * z_DelAtmMass(k) * r_Temp(k+1)
      r_Temp(k-1) = r_Temp(k-1) * EneSumB / EneSum
      r_Temp(k  ) = r_Temp(k  ) * EneSumB / EneSum
      r_Temp(k+1) = r_Temp(k+1) * EneSumB / EneSum
    end if
  end do

  r_DelTemp = r_Temp - r_TempB


end subroutine Filter

!------------------------------------------------------------------------------

subroutine OutputTendencies( &
  & OutTendencyNcFN, &
  & DelTime, &
  & kmax, r_Press, &
  & r_DelTempRC, &
  & z_DTempDtLW, z_DTempDtSW, &
  & r_DelTempFilter,  &
  & DelSurfTemp  &
  & )

  use vtype_module

  use ni3_module

  implicit none

  character(*), intent(in ) :: OutTendencyNcFN
  real(DP)    , intent(in ) :: DelTime
  integer     , intent(in ) :: kmax
  real(DP)    , intent(in ) :: r_Press        (0:kmax)
  real(DP)    , intent(in ) :: r_DelTempRC    (0:kmax)
  real(DP)    , intent(in ) :: z_DTempDtLW    (1:kmax)
  real(DP)    , intent(in ) :: z_DTempDtSW    (1:kmax)
  real(DP)    , intent(in ) :: r_DelTempFilter(0:kmax)
  real(DP)    , intent(in ) :: DelSurfTemp


  ! Local variables
  !
  integer :: NcID
  character(extstr) :: Name
  character(extstr) :: StdName
  character(extstr) :: Units
  integer, parameter :: NDims1 = 1
  character(extstr)  :: a_DimName1(NDims1)
  character(extstr)  :: a_DimName0(0)

  real(DP), parameter :: MissingValue = -1.0d100

  real(DP)            :: r_DTempDtConv  (0:kmax)
  real(DP)            :: r_DTempDtSW    (0:kmax)
  real(DP)            :: r_DTempDtLW    (0:kmax)
  real(DP)            :: r_DTempDtFilter(0:kmax)
  real(DP)            :: DSurfTempDt


!!$  r_DTempDtSW(0     ) = 0.0d0
  r_DTempDtSW(0     ) = MissingValue
  r_DTempDtSW(1:kmax) = z_DTempDtSW
!!$  r_DTempDtLW(0     ) = 0.0d0
  r_DTempDtLW(0     ) = MissingValue
  r_DTempDtLW(1:kmax) = z_DTempDtLW

  r_DTempDtConv       = r_DelTempRC / DelTime - r_DTempDtSW - r_DTempDtLW
  r_DTempDtConv(0)    = MissingValue

  r_DTempDtFilter     = r_DelTempFilter / DelTime
  r_DTempDtFilter(0)  = MissingValue

  DSurfTempDt         = DelSurfTemp / DelTime


  call ni3_open( OutTendencyNcFN, "new", NcID )

  Name    = "Press"
  StdName = "plev"
  Units   = "Pa"
  call ni3_set_dim( NcID, name, NI3_DOUBLE, r_Press, &
    & stdname = stdname, units  = units )


  a_DimName1( 1 ) = "Press"

  Name    = "DTempDtSW"
  StdName = "radiative heating rate, short wave"
  Units   = "K s-1"
  call ni3_def_var( NcID, name, NI3_DOUBLE, NDims1, a_DimName1, &
    & stdname = stdname, units = units, fillvalue = MissingValue )
  call ni3_put_var( NcID, name, r_DTempDtSW )
  call ni3_put_att( NcID, name, "missing_value", MissingValue )

  Name    = "DTempDtLW"
  StdName = "radiative heating rate, long wave"
  Units   = "K s-1"
  call ni3_def_var( NcID, name, NI3_DOUBLE, NDims1, a_DimName1, &
    & stdname = stdname, units = units, fillvalue = MissingValue )
  call ni3_put_var( NcID, name, r_DTempDtLW )
  call ni3_put_att( NcID, name, "missing_value", MissingValue )

  Name    = "DTempDtConv"
  StdName = "convective heating rate"
  Units   = "K s-1"
  call ni3_def_var( NcID, name, NI3_DOUBLE, NDims1, a_DimName1, &
    & stdname = stdname, units = units, fillvalue = MissingValue )
  call ni3_put_var( NcID, name, r_DTempDtConv )
  call ni3_put_att( NcID, name, "missing_value", MissingValue )

  Name    = "DTempDtFilter"
  StdName = "heating rate by filter"
  Units   = "K s-1"
  call ni3_def_var( NcID, name, NI3_DOUBLE, NDims1, a_DimName1, &
    & stdname = stdname, units = units, fillvalue = MissingValue )
  call ni3_put_var( NcID, name, r_DTempDtFilter )
  call ni3_put_att( NcID, name, "missing_value", MissingValue )

  Name    = "DSurfTempDt"
  StdName = "surface heating rate"
  Units   = "K s-1"
  call ni3_def_var( NcID, name, NI3_DOUBLE, 0, a_DimName0, &
    & stdname = stdname, units = units )
  call ni3_put_var( NcID, name, DSurfTempDt )
!!$  call ni3_put_att( NcID, name, "missing_value", MissingValue )

  call ni3_close( NcID )


end subroutine OutputTendencies

!------------------------------------------------------------------------------

subroutine CalcMeanMolWt( &
  & kmax, NMolNum, m_MolNum, rm_VMR, r_VMRNonRadAct, &
  & r_MeanMolWt, &
  & FlagOutput &
  & )

  use vtype_module

  use rad_planetconst, only : MolWtNonRadAct
  use hitranconst, only : m_HITRANMolWt

  implicit none

  integer , intent(in ) :: kmax
  integer , intent(in ) :: NMolNum
  integer , intent(in ) :: m_MolNum      (1:NMolNum)
  real(DP), intent(in ) :: rm_VMR        (0:kmax,1:NMolNum)
  real(DP), intent(in ) :: r_VMRNonRadAct(0:kmax)
  real(DP), intent(out) :: r_MeanMolWt   (0:kmax)
  logical , intent(in ) :: FlagOutput


  ! Local variables
  !
  real(dp) :: r_SumVMR(0:kmax)

  integer  :: k
  integer  :: m


  r_MeanMolWt = 0.0d0
  r_MeanMolWt = r_MeanMolWt + r_VMRNonRadAct * MolWtNonRadAct
  r_SumVMR = 0.0d0
  r_SumVMR = r_SumVMR + r_VMRNonRadAct
  do m = 1, NMolNum
    r_MeanMolWt = r_MeanMolWt + rm_VMR(:,m) * m_HITRANMolWt(m_MolNum(m))
    r_SumVMR    = r_SumVMR    + rm_VMR(:,m)
  end do
  ! Now, this is not normalized.
!!$  r_MeanMolWt = r_MeanMolWt / r_SumVMR


  if ( FlagOutput ) then
    write( 6, * ) 'Mean molecular weight'
    do k = 0, kmax
      write( 6, * ) k, r_MeanMolWt(k)
    end do
  end if


end subroutine CalcMeanMolWt

!------------------------------------------------------------------------------
!!$
!!$subroutine ConvectionTrial( &
!!$  & kmax, NMolNum, r_Press, r_MeanMolWt, &
!!$  & r_Temp, rm_VMR &
!!$  & )
!!$
!!$  use vtype_module
!!$  use constants0, only : GasRUniv
!!$  use rad_planetconst, only : &
!!$    & Cp,       &
!!$    & MolWtH2O, &
!!$    & CpH2OVap
!!$
!!$  implicit none
!!$
!!$  integer , intent(in   ) :: kmax
!!$  integer , intent(in   ) :: NMolNum
!!$  real(DP), intent(in   ) :: r_Press(0:kmax)
!!$  real(DP), intent(in   ) :: r_MeanMolWt(0:kmax)
!!$  real(DP), intent(inout) :: r_Temp     (0:kmax)
!!$  real(DP), intent(inout) :: rm_VMR     (0:kmax,1:NMolNum)
!!$
!!$
!!$  ! Local variables
!!$  !
!!$  real(DP), parameter :: LatentHeat = 2.5d6    ! J kg-1
!!$
!!$  real(DP) :: CPPerMol
!!$  real(DP) :: CpH2OVapPerMol
!!$  real(DP) :: CPTotPerMol
!!$  real(DP) :: LatentHeatPerMol
!!$  real(DP) :: PressH2OVapSat
!!$  real(DP) :: DPressH2OVapSatDTemp
!!$  real(DP) :: VMRSat
!!$  real(DP) :: VMRSave
!!$  real(DP) :: DelTemp
!!$  real(DP) :: TempTmp
!!$
!!$  real(DP) :: LOCALPressH2OVapSat
!!$  real(DP) :: LOCALDPressH2OVapSatDTemp
!!$
!!$  logical  :: FlagMoistConvection
!!$  logical  :: FlagConnectToSurface
!!$
!!$  integer  :: k
!!$
!!$
!!$  FlagMoistConvection  = .true.
!!$  FlagConnectToSurface = .true.
!!$
!!$
!!$  LatentHeatPerMol = LatentHeat * MolWtH2O
!!$  CpH2OVapPerMol   = CpH2OVap * MolWtH2O
!!$
!!$  PressH2OVapSat   = LOCALPressH2OVapSat( LatentHeatPerMol, r_Temp(0) )
!!$  VMRSat           = PressH2OVapSat / r_Press(0)
!!$  rm_VMR(0,1)      = VMRSat
!!$
!!$  VMRSave = rm_VMR(0,1)
!!$
!!$  do k = 1, kmax
!!$
!!$    if ( FlagMoistConvection ) then
!!$      ! moist convection
!!$
!!$      CpPerMol = Cp * r_MeanMolWt(k-1)
!!$      PressH2OVapSat = LOCALPressH2OVapSat( LatentHeatPerMol, r_Temp(k-1) )
!!$      DPressH2OVapSatDTemp = LOCALDPressH2OVapSatDTemp( LatentHeatPerMol, r_Temp(k-1) )
!!$      VMRSat = PressH2OVapSat / r_Press(k-1)
!!$      CpTotPerMol = CpPerMol * ( 1.0d0 - VMRSat ) + CpH2OVapPerMol * VMRSat
!!$      DelTemp = ( GasRUniv * r_Temp(k-1) + LatentHeatPerMol * VMRSat ) &
!!$        & / ( CPTotPerMol + LatentHeatPerMol / r_Press(k-1) * DPressH2OVapSatDTemp ) &
!!$        & * log( r_Press(k) / r_Press(k-1) )
!!$      !
!!$      if ( FlagConnectToSurface ) then
!!$        TempTmp = r_Temp(k-1) + DelTemp
!!$        if ( r_Temp(k) < TempTmp ) then
!!$          r_Temp(k)     = TempTmp
!!$          rm_VMR(k-1,1) = VMRSat
!!$        else
!!$          FlagConnectToSurface = .false.
!!$        end if
!!$      end if
!!$      if ( .not. FlagConnectToSurface ) then
!!$        rm_VMR(k-1,1) = VMRSave
!!$        if ( rm_VMR(k-1,1) > VMRSat ) then
!!$          rm_VMR(k-1,1) = VMRSat
!!$        end if
!!$        VMRSave = rm_VMR(k-1,1)
!!$      end if
!!$
!!$    else
!!$      ! dry convection
!!$!      GasConst = GasRUniv / r_MeanMolWt(k-1)
!!$!      DelTemp = GasConst * TempLL / Cp * log( z_Press(k) / PressLL )
!!$      CpPerMol = Cp * r_MeanMolWt(k-1)
!!$      DelTemp = GasRUniv * r_Temp(k-1) / CpPerMol * log( r_Press(k) / r_Press(k-1) )
!!$      if ( FlagConnectToSurface ) then
!!$        TempTmp = r_Temp(k-1) + DelTemp
!!$        if ( r_Temp(k) < TempTmp ) then
!!$          r_Temp(k)     = TempTmp
!!$        else
!!$          FlagConnectToSurface = .false.
!!$        end if
!!$      end if
!!$    end if
!!$    !
!!$    !
!!$    if ( k == kmax ) then
!!$      rm_VMR(k,1) = rm_VMR(k-1,1)
!!$    end if
!!$  end do
!!$
!!$
!!$end subroutine ConvectionTrial

!------------------------------------------------------------------------------

!!$function LOCALPressH2OVapSat( LatentHeatPerMol, Temp ) result( PressH2OVapSat )
!!$
!!$  use vtype_module
!!$  use constants0, only : GasRUniv
!!$
!!$  real(DP), intent(in ) :: LatentHeatPerMol
!!$  real(DP), intent(in ) :: Temp
!!$
!!$  real(DP) :: PressH2OVapSat
!!$
!!$
!!$  PressH2OVapSat = 1.4d11 * exp( - LatentHeatPerMol / ( GasRUniv * Temp ) )
!!$
!!$end function LOCALPressH2OVapSat

!------------------------------------------------------------------------------

!!$function LOCALDPressH2OVapSatDTemp( LatentHeatPerMol, Temp ) result( DPressH2OVapSatDTemp )
!!$
!!$  use vtype_module
!!$  use constants0, only : GasRUniv
!!$
!!$  real(DP), intent(in ) :: LatentHeatPerMol
!!$  real(DP), intent(in ) :: Temp
!!$
!!$  real(DP) :: DPressH2OVapSatDTemp
!!$
!!$
!!$  DPressH2OVapSatDTemp = &
!!$    & 1.4d11 * exp( - LatentHeatPerMol / ( GasRUniv * Temp ) ) &
!!$    & * LatentHeatPerMol / ( GasRUniv * Temp**2 )
!!$
!!$end function LOCALDPressH2OVapSatDTemp

!------------------------------------------------------------------------------

subroutine Diffusion( &
  & DelTime, &
  & kmax, r_Press, z_Press, r_MeanMolWt, &
  & z_Temp &
  & )

  use vtype_module
  use constants0, only : GasRUniv
  use rad_planetconst, only : Grav

  implicit none

  real(DP), intent(in   ) :: DelTime
  integer , intent(in   ) :: kmax
  real(DP), intent(in   ) :: r_Press(0:kmax)
  real(DP), intent(in   ) :: z_Press(1:kmax)
  real(DP), intent(in   ) :: r_MeanMolWt(0:kmax)
  real(DP), intent(inout) :: z_Temp     (1:kmax)


  ! Local variables
  !
  real(DP) :: DiffCoef = 0.0d0
!!$  real(DP) :: DiffCoef = 1.0d0
  real(DP) :: r_Flux(0:kmax)
  real(DP) :: GasConst
  real(DP) :: Temp
  real(DP) :: Rho

  integer  :: k


  k = 0
  r_Flux(k) = 0.0d0
  do k = 0+1, kmax-1
    GasConst = GasRUniv / r_MeanMolWt(k)
    Temp = ( z_Temp(k) + z_Temp(k+1) ) / 2.0d0
    Rho = r_Press(k) / ( GasConst * Temp )
    r_Flux(k) = &
      &   DiffCoef &
      & * ( z_Temp(k+1) - z_Temp(k) ) / ( z_Press(k+1) - z_Press(k) ) &
      & * Rho**2
  end do
  k = kmax
  r_Flux(k) = 0.0d0


  do k = 1, kmax
    z_Temp(k) = z_Temp(k) &
      & + Grav * ( r_Flux(k) - r_Flux(k-1) ) / ( r_Press(k) - r_Press(k-1) ) &
      &   * DelTime
  end do


end subroutine Diffusion
