program main

  use vtype_module

  use ni3_module

  use fi_module

  implicit none

  integer                 :: InNcID
  integer                 :: OutNcID
  character(extstr)       :: Mode
  character(extstr)       :: Name
!!$  character(extstr)       :: StdName
!!$  character(extstr)       :: LongName
!!$  character(extstr)       :: Units
  integer                 :: NDims
  character(extstr), allocatable :: a_DimNames(:)

  integer                 :: NAve

  integer                 :: NLev
  integer                 :: NWaveNum
  real(dp), allocatable   :: p_Press  (:)
  real(dp), allocatable   :: p_PressLC(:)
  real(dp), allocatable   :: w_WaveNum(:)
  real(dp), allocatable   :: pw_UwFlux        (:,:)
  real(dp), allocatable   :: pw_DwFlux        (:,:)
  real(dp), allocatable   :: pw_DRadFluxDPress(:,:)
  real(dp), allocatable   :: pw_DelOptDep     (:,:)

  integer                 :: NWaveNumOut
  real(dp), allocatable   :: w_WaveNumOut(:)
  real(dp), allocatable   :: pw_UwFluxOut        (:,:)
  real(dp), allocatable   :: pw_DwFluxOut        (:,:)
  real(dp), allocatable   :: pw_DRadFluxDPressOut(:,:)
  real(dp), allocatable   :: pw_DelOptDepOut     (:,:)

  integer :: NAveData

  character(128) :: ctlfn = 'runave.conf'

  character(128) :: InNcFn
  character(128) :: OutNcFn
  character(128) :: PressName
  character(128) :: WaveNumName
  character(128) :: UwFluxName
  character(128) :: DwFluxName
  character(128) :: FluxDivName
  character(128) :: DelOptDepName

  integer :: FileUnit
  integer :: iostat

  integer :: i
  integer :: j
  integer :: l


  namelist /ave_nml/ &
    & InNcFn, OutNcFn, NAve

  InNcFn      = ''
  OutNcFn     = ''

  NAve        = 1

  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = ave_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist, ave_nml, is something wrong.'
  end if
  write( 6, nml = ave_nml )
  !
  close( FileUnit )

  PressName     = "PressLC"
  WaveNumName   = "WaveNum"
  UwFluxName    = "RadUwFlux"
  DwFluxName    = "RadDwFlux"
  FluxDivName   = "DRadFluxDPress"
  DelOptDepName = "DelOptDep"
  !
  ! Input
  !
  Mode = "read"
  call ni3_open( InNcFN, Mode, InNcID )

  call ni3_inq_dimlen( InNcID, "Press"  , NLev     )
  call ni3_inq_dimlen( InNcID, "WaveNum", NWaveNum )

  allocate( p_Press          ( NLev     ) )
  allocate( p_PressLC        ( NLev-1   ) )
  allocate( w_WaveNum        ( NWaveNum ) )
  allocate( pw_UwFlux        ( NLev    , NWaveNum ) )
  allocate( pw_DwFlux        ( NLev    , NWaveNum ) )
  allocate( pw_DRadFluxDPress( NLev-1  , NWaveNum ) )
  allocate( pw_DelOptDep     ( NLev-1  , NWaveNum ) )

  call ni3_get_var( InNcID, "Press"      , p_Press           )
  call ni3_get_var( InNcID, "PressLC"    , p_PressLC         )
  call ni3_get_var( InNcID, WaveNumName  , w_WaveNum         )
  call ni3_get_var( InNcID, UwFluxName   , pw_UwFlux         )
  call ni3_get_var( InNcID, DwFluxName   , pw_DwFlux         )
  call ni3_get_var( InNcID, FluxDivName  , pw_DRadFluxDPress )
  call ni3_get_var( InNcID, DelOptDepName, pw_DelOptDep      )


  NWaveNumOut = NWaveNum/NAve
  if ( mod( NWaveNum, NAve ) >= 1 ) then
     NWaveNumOut = NWaveNumOut + 1
  end if
  allocate( w_WaveNumOut( NWaveNumOut ) )
  allocate( pw_UwFluxOut        ( NLev  , NWaveNumOut ) )
  allocate( pw_DwFluxOut        ( NLev  , NWaveNumOut ) )
  allocate( pw_DRadFluxDPressOut( NLev-1, NWaveNumOut ) )
  allocate( pw_DelOptDepOut     ( NLev-1, NWaveNumOut ) )
  l = 1
  do i = 1, NWaveNum, NAve
     w_WaveNumOut(l) = 0.0d0
     pw_UwFluxOut        (:,l) = 0.0d0
     pw_DwFluxOut        (:,l) = 0.0d0
     pw_DRadFluxDPressOut(:,l) = 0.0d0
     pw_DelOptDepOut     (:,l) = 0.0d0
     NAveData = 0
     do j = 1, NAve
        if ( i+j-1 > NWaveNum ) exit
        w_WaveNumOut        (l)   = w_WaveNumOut(l) &
             & + w_WaveNum(i+j-1)
        pw_UwFluxOut(:,l) = pw_UwFluxOut(:,l) &
             & + pw_UwFlux(:,i+j-1) 
        pw_DwFluxOut(:,l) = pw_DwFluxOut(:,l) &
             & + pw_DwFlux(:,i+j-1) 
        pw_DRadFluxDPressOut(:,l) = pw_DRadFluxDPressOut(:,l) &
             & + pw_DRadFluxDPress(:,i+j-1) 
        pw_DelOptDepOut(:,l) = pw_DelOptDepOut(:,l) &
             & + pw_DelOptDep(:,i+j-1) 
        NAveData = NAveData + 1
     end do
     w_WaveNumOut        (l)   = w_WaveNumOut        (l)   / NAveData
     pw_UwFluxOut        (:,l) = pw_UwFluxOut        (:,l) / NAveData
     pw_DwFluxOut        (:,l) = pw_DwFluxOut        (:,l) / NAveData
     pw_DRadFluxDPressOut(:,l) = pw_DRadFluxDPressOut(:,l) / NAveData
     pw_DelOptDepOut     (:,l) = pw_DelOptDepOut     (:,l) / NAveData
     l = l + 1
  end do


  ! Output

  call ni3_open( OutNcFN, "new", OutNcID )

  Name     = "Press"
  call ni3_set_dim( OutNcID, Name, NI3_DOUBLE, p_Press )
  call ni3_cp_atts( InNcID, OutNcID, Name, Name )

  Name     = PressName
  call ni3_set_dim( OutNcID, Name, NI3_DOUBLE, p_PressLC )
  call ni3_cp_atts( InNcID, OutNcID, Name, Name )

  Name     = WaveNumName
  call ni3_set_dim( OutNcID, Name, NI3_DOUBLE, w_WaveNumOut )
  call ni3_cp_atts( InNcID, OutNcID, Name, Name )

  NDims = 2
  allocate( a_DimNames( NDims ) )
  !
  Name     = UwFluxName
  a_DimNames( 1 ) = "Press"
  a_DimNames( 2 ) = "WaveNum"
  call ni3_def_var( OutNcID, Name, NI3_DOUBLE, NDims, a_DimNames )
  call ni3_cp_atts( InNcID, OutNcID, Name, Name )
  !
  Name     = DwFluxName
  a_DimNames( 1 ) = "Press"
  a_DimNames( 2 ) = "WaveNum"
  call ni3_def_var( OutNcID, Name, NI3_DOUBLE, NDims, a_DimNames )
  call ni3_cp_atts( InNcID, OutNcID, Name, Name )
  !
  Name     = FluxDivName
  a_DimNames( 1 ) = "PressLC"
  a_DimNames( 2 ) = "WaveNum"
  call ni3_def_var( OutNcID, Name, NI3_DOUBLE, NDims, a_DimNames )
  call ni3_cp_atts( InNcID, OutNcID, Name, Name )
  !
  Name     = DelOptDepName
  a_DimNames( 1 ) = "PressLC"
  a_DimNames( 2 ) = "WaveNum"
  call ni3_def_var( OutNcID, Name, NI3_DOUBLE, NDims, a_DimNames )
  call ni3_cp_atts( InNcID, OutNcID, Name, Name )
  !
  deallocate( a_DimNames )

  Name     = UwFluxName
  call ni3_put_var( OutNcID, Name, pw_UwFluxOut )
  Name     = DwFluxName
  call ni3_put_var( OutNcID, Name, pw_DwFluxOut )
  Name     = FluxDivName
  call ni3_put_var( OutNcID, Name, pw_DRadFluxDPressOut )
  Name     = DelOptDepName
  call ni3_put_var( OutNcID, Name, pw_DelOptDepOut )


  call ni3_close( InNcID  )
  call ni3_close( OutNcID )


end program main
